@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -53,23 +53,23 @@ typedef struct
 
 const UgEntry  Test1Entry[] =
 {
-	{"type", offsetof (Test1, type), UG_ENTRY_INT, NULL, NULL},
+	{"type", offsetof(Test1, type), UG_ENTRY_INT, NULL, NULL},
 	{NULL}
 };
 
-void  test1_init (Test1* t1);
+void  test1_init(Test1* t1);
 
 const UgDataInfo  Test1Info =
 {
 	"Test1",
-	sizeof (Test1),
-	Test1Entry,		// UgEntry
+	sizeof(Test1),
 	(UgInitFunc)   test1_init,
 	(UgFinalFunc)  NULL,
-	(UgAssignFunc) NULL
+	(UgAssignFunc) NULL,
+	Test1Entry,		// UgEntry
 };
 
-void  test1_init (Test1* t1)
+void  test1_init(Test1* t1)
 {
 	t1->info = &Test1Info;
 	t1->type = 1;
@@ -88,23 +88,23 @@ typedef struct
 
 const UgEntry  Test2Entry[] =
 {
-	{"type", offsetof (Test2, type), UG_ENTRY_INT, NULL, NULL},
+	{"type", offsetof(Test2, type), UG_ENTRY_INT, NULL, NULL},
 	{NULL}
 };
 
-void  test2_init (Test2* t2);
+void  test2_init(Test2* t2);
 
 const UgDataInfo  Test2Info =
 {
 	"Test2",
-	sizeof (Test2),
-	Test2Entry,		// UgEntry
+	sizeof(Test2),
 	(UgInitFunc)   test2_init,
 	(UgFinalFunc)  NULL,
-	(UgAssignFunc) NULL
+	(UgAssignFunc) NULL,
+	Test2Entry,		// UgEntry
 };
 
-void  test2_init (Test2* t2)
+void  test2_init(Test2* t2)
 {
 	t2->info = &Test2Info;
 	t2->type = 2;
@@ -123,23 +123,23 @@ typedef struct
 
 const UgEntry  Test3Entry[] =
 {
-	{"type", offsetof (Test3, type), UG_ENTRY_INT, NULL, NULL},
+	{"type", offsetof(Test3, type), UG_ENTRY_INT, NULL, NULL},
 	{NULL}
 };
 
-void  test3_init (Test3* t3);
+void  test3_init(Test3* t3);
 
 const UgDataInfo  Test3Info =
 {
 	"Test3",
-	sizeof (Test3),
-	Test3Entry,		// UgEntry
+	sizeof(Test3),
 	(UgInitFunc)   test3_init,
 	(UgFinalFunc)  NULL,
-	(UgAssignFunc) NULL
+	(UgAssignFunc) NULL,
+	Test3Entry,		// UgEntry
 };
 
-void  test3_init (Test3* t3)
+void  test3_init(Test3* t3)
 {
 	t3->info = &Test3Info;
 	t3->type = 3;
@@ -154,20 +154,20 @@ UgEntry	InfoCustomEntry[] = {
 	{NULL}
 };
 
-void  test_info (UgInfo* info)
+void  test_info(UgInfo* info)
 {
 	void*  data;
 
-	puts ("\n--- test_info:");
-	data = ug_info_realloc (info, &Test2Info);
-	printf ("ug_info_realloc (Test2Info) : %d\n", ((Test2*)data)->type);
-	data = ug_info_realloc (info, &Test1Info);
-	printf ("ug_info_realloc (Test1Info) : %d\n", ((Test1*)data)->type);
-	data = ug_info_realloc (info, &Test3Info);
-	printf ("ug_info_realloc (Test3Info) : %d\n", ((Test3*)data)->type);
+	puts("\n--- test_info:");
+	data = ug_info_realloc(info, &Test2Info);
+	printf("ug_info_realloc (Test2Info) : %d\n", ((Test2*)data)->type);
+	data = ug_info_realloc(info, &Test1Info);
+	printf("ug_info_realloc (Test1Info) : %d\n", ((Test1*)data)->type);
+	data = ug_info_realloc(info, &Test3Info);
+	printf("ug_info_realloc (Test3Info) : %d\n", ((Test3*)data)->type);
 }
 
-void  parse_info (UgInfo* info)
+void  parse_info(UgInfo* info)
 {
 	int     code;
 	UgJson	json;
@@ -187,107 +187,107 @@ void  parse_info (UgInfo* info)
 	};
 
 	// UgRegistry is used by ug_json_parse_info() and ug_json_parse_info_entry()
-	ug_registry_init (&registry);
-	ug_registry_add (&registry, &Test2Info);
-	ug_registry_add (&registry, &Test3Info);
-	ug_registry_add (&registry, &Test1Info);
-//	ug_registry_sort (&registry);
+	ug_registry_init(&registry);
+	ug_registry_add(&registry, &Test2Info);
+	ug_registry_add(&registry, &Test3Info);
+	ug_registry_add(&registry, &Test1Info);
+//	ug_registry_sort(&registry);
 	// ug_json_parse_info() must use default UgInfoKeys.
-	ug_info_set_registry (&registry);
+	ug_info_set_registry(&registry);
 
-	ug_json_init (&json);
-	ug_json_begin_parse (&json);
+	ug_json_init(&json);
+	ug_json_begin_parse(&json);
 #if 1
 	// method 1: use UgEntry to parse start of object
-	ug_json_push (&json, ug_json_parse_entry, info, InfoCustomEntry);
+	ug_json_push(&json, ug_json_parse_entry, info, InfoCustomEntry);
 #else
 	// method 2: push ug_json_parse_info() to parse start of object
-	ug_json_push (&json, ug_json_parse_info, info, &registry);
+	ug_json_push(&json, ug_json_parse_info, info, &registry);
 #endif
-	code = ug_json_parse (&json, json_string, -1);
-	ug_json_end_parse (&json);
-	ug_json_final (&json);
-	printf ("ug_json_parse response %d\n", code);
+	code = ug_json_parse(&json, json_string, -1);
+	ug_json_end_parse(&json);
+	ug_json_final(&json);
+	printf("ug_json_parse response %d\n", code);
 
-	ug_info_set_registry (NULL);
-	ug_registry_final (&registry);
+	ug_info_set_registry(NULL);
+	ug_registry_final(&registry);
 }
 
-void  dump_info (UgInfo* info)
+void  dump_info(UgInfo* info)
 {
 	UgPair*	cur;
 	UgPair*	end;
 
 	for (cur = info->at, end = info->at + info->length;  cur < end;  cur++) {
 		if (cur->key == NULL) {
-			puts ("NULL");
+			puts("NULL");
 			continue;
 		}
 
-		printf ("%s", ((UgDataInfo*)cur->key)->name);
+		printf("%s", ((UgDataInfo*)cur->key)->name);
 		if (cur->data)
-			printf (" : %d", ((Test1*)cur->data)->type);
-		puts ("");
+			printf(" : %d", ((Test1*)cur->data)->type);
+		puts("");
 	}
 }
 
 // UgBufferFunc
-static int  buffer_to_file (UgBuffer* buffer)
+static int  buffer_to_file(UgBuffer* buffer)
 {
 	FILE*   file = buffer->data;
 
-	printf ("write %d bytes to file\n", ug_buffer_length (buffer));
-	fwrite (buffer->beg, 1, ug_buffer_length(buffer), file);
+	printf("write %d bytes to file\n", ug_buffer_length(buffer));
+	fwrite(buffer->beg, 1, ug_buffer_length(buffer), file);
 	buffer->cur = buffer->beg;
 	return 0;
 }
 
-void  write_info_to_file (UgInfo* info, char* filename)
+void  write_info_to_file(UgInfo* info, char* filename)
 {
 	UgJson   json;
 	FILE*    file;
 	UgBuffer buffer;
 
-	file = fopen (filename, "w");
-	ug_buffer_init (&buffer, 128);
+	file = fopen(filename, "w");
+	ug_buffer_init(&buffer, 128);
 	buffer.more = buffer_to_file;
 	buffer.data = file;
 
-	ug_json_init (&json);
-	ug_json_begin_write (&json, UG_JSON_FORMAT_INDENT, &buffer);
+	ug_json_init(&json);
+	ug_json_begin_write(&json, UG_JSON_FORMAT_INDENT, &buffer);
 #if 1
 	// method 1: use UgEntry to write start of object
-	ug_json_write_entry (&json, info, InfoCustomEntry);
+	ug_json_write_entry(&json, info, InfoCustomEntry);
 #else
 	// method 2: call ug_json_write_object_head() to write start of object
-	ug_json_write_object_head (&json);
-	ug_json_write_info (&json, info);
-	ug_json_write_object_tail (&json);
+	ug_json_write_object_head(&json);
+	ug_json_write_info(&json, info);
+	ug_json_write_object_tail(&json);
 #endif
-	ug_json_end_write (&json);
-	ug_json_final (&json);
+	ug_json_end_write(&json);
+	ug_json_final(&json);
 
-	ug_buffer_clear (&buffer, 1);
-	fclose (file);
+	ug_buffer_clear(&buffer, 1);
+	fclose(file);
 }
 
 // ----------------------------------------------------------------------------
 // main
 
-int   main (void)
+int   main(void)
 {
 	UgInfo  info;
 
-	puts ("\n--- test UgInfo functions:");
-	ug_info_init (&info, 8, 2);
+	puts("\n--- test UgInfo functions:");
+	ug_info_init(&info, 8, 2);
 #if 0
-	test_info (&info);
+	test_info(&info);
 #else
-	parse_info (&info);
+	parse_info(&info);
 #endif
-	dump_info (&info);
-	write_info_to_file (&info, "test-info.json");
-	ug_info_final (&info);
+	dump_info(&info);
+	write_info_to_file(&info, "test-info.json");
+	ug_info_final(&info);
 
 	return 0;
 }
