@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2011-2018 by C.H. Huang
+ *   Copyright (C) 2011-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -40,6 +40,8 @@
 #include <UgStdio.h>
 #include <UgetA2cf.h>
 
+#define INFO_HASH_LEN_MAX    8192
+
 enum {
 	ENDIAN_UNKNOWN,
 	ENDIAN_BE,
@@ -136,7 +138,7 @@ static void  set_bit (uint8_t* bytes, uint32_t nth_bit);
 
 // ----------------------------------------------------------------------------
 
-#define A2CF_LAST_PIECE_LEN(a2cf)  ((a2cf)->total_size & ((a2cf)->piece_len-1))
+#define A2CF_LAST_PIECE_LEN(a2cf)  ((a2cf)->total_size % (a2cf)->piece_len)
 
 static UgetA2cfPiece*  a2cf_piece_new (uint32_t length)
 {
@@ -356,21 +358,29 @@ int  uget_a2cf_load (UgetA2cf* a2cf, const char* filename)
 	FILE*    file;
 	uint32_t index;
 	uint32_t n_pieces;
+	uint32_t bitfield_len;
 
 	init_endian_type ();
 
 	file = ug_fopen (filename, "rb");
 	if (file == NULL)
 		return FALSE;
+	// version
 	ug_fread (file, (char*)&a2cf->ver, 2);
 	ug_fread (file, (char*)&a2cf->ext, 4);
 	a2cf->ver = uint16_from_be (a2cf->ver);
 	a2cf->ext = uint32_from_be (a2cf->ext);
+	// check file version - uGet only support version 1
+	if (a2cf->ver != 1)
+		goto failed;
+
 	// info hash
 	a2cf->info_hash_len = 0;
 	ug_fread (file, (char*)&a2cf->info_hash_len, 4);
 	a2cf->info_hash_len = uint32_from_be (a2cf->info_hash_len);
-	if (a2cf->info_hash_len == 0)
+	if (a2cf->info_hash_len > INFO_HASH_LEN_MAX)
+		goto failed;
+	else if (a2cf->info_hash_len == 0)
 		a2cf->info_hash = NULL;
 	else {
 		a2cf->info_hash = ug_malloc (a2cf->info_hash_len);
@@ -383,33 +393,34 @@ int  uget_a2cf_load (UgetA2cf* a2cf, const char* filename)
 	a2cf->piece_len = uint32_from_be (a2cf->piece_len);
 	a2cf->total_len = uint64_from_be (a2cf->total_len);
 	a2cf->upload_len = uint64_from_be (a2cf->upload_len);
-	// bit field
-	if (ug_fread (file, (char*)&a2cf->bitfield_len, 4) != 4) {
-		fclose (file);
-		return FALSE;
-	}
+
+	// bitfield
+	if (ug_fread (file, (char*)&a2cf->bitfield_len, 4) != 4)
+		goto failed;
 	a2cf->bitfield_len = uint32_from_be (a2cf->bitfield_len);
-	if (a2cf->bitfield_len == 0) {
+	// check bitfield_len
+	bitfield_len  = (uint32_t)(a2cf->total_len / (8 * a2cf->piece_len));
+	bitfield_len += (uint32_t)(a2cf->total_len % (8 * a2cf->piece_len)) ? 1 : 0;
+	if (a2cf->bitfield_len == 0 || a2cf->bitfield_len != bitfield_len) {
 		a2cf->bitfield = NULL;
-		return FALSE;
+		goto failed;
 	}
 	else {
 		a2cf->bitfield = ug_malloc (a2cf->bitfield_len);
 		if (ug_fread (file, a2cf->bitfield, a2cf->bitfield_len) != a2cf->bitfield_len)
-		{
-			fclose (file);
-			return FALSE;
-		}
+			goto failed;
 	}
 
+	// The number of in-flight pieces.
 	n_pieces = 0;
 	ug_fread (file, (char*)&n_pieces, 4);
 	n_pieces = uint32_from_be (n_pieces);
 
-	// piece.index_end
+	// piece.index_end - calculate number of the last piece
 	a2cf->piece.index_end = (uint32_t) (a2cf->total_len / a2cf->piece_len) +
-			( (a2cf->total_len & (a2cf->piece_len-1)) ? 1 : 0 );
-	// load pieces
+			( (a2cf->total_len % a2cf->piece_len) ? 1 : 0 );
+
+	// load in-flight pieces
 	for (index = 0;  index < n_pieces;  index++) {
 		piece = a2cf_piece_new (a2cf->piece_len);
 		if (a2cf_piece_read (piece, file) == FALSE) {
@@ -421,6 +432,10 @@ int  uget_a2cf_load (UgetA2cf* a2cf, const char* filename)
 
 	fclose (file);
 	return TRUE;
+
+failed:
+	fclose(file);
+	return FALSE;
 }
 
 int   uget_a2cf_save (UgetA2cf* a2cf, const char* filename)
@@ -491,7 +506,7 @@ int   uget_a2cf_lack (UgetA2cf* a2cf, uint64_t* beg, uint64_t* end)
 		return FALSE;
 
 	index     = (uint32_t) (beg[0] / a2cf->piece_len);
-	piece_beg = (uint32_t) (beg[0] & (a2cf->piece_len-1));
+	piece_beg = (uint32_t) (beg[0] % a2cf->piece_len);
 
 	// find begin
 	for (;  index < a2cf->piece.index_end;  index++) {
@@ -517,6 +532,7 @@ int   uget_a2cf_lack (UgetA2cf* a2cf, uint64_t* beg, uint64_t* end)
 			}
 		}
 		beg[0] = (uint64_t)index * a2cf->piece_len + piece_beg;
+		// if current piece is the last piece
 		if (index == a2cf->piece.index_end - 1) {
 			end[0] = a2cf->total_len;
 			return TRUE;
@@ -595,8 +611,8 @@ uint64_t  uget_a2cf_fill (UgetA2cf* a2cf, uint64_t beg, uint64_t end)
 
 	index_beg = (uint32_t) (beg / a2cf->piece_len);
 	index_end = (uint32_t) (end / a2cf->piece_len);
-	piece_beg = beg & (a2cf->piece_len-1);
-	piece_end = end & (a2cf->piece_len-1);
+	piece_beg = (uint32_t) (beg % a2cf->piece_len);
+	piece_end = (uint32_t) (end % a2cf->piece_len);
 
 	// first piece or only 1 piece
 	if (index_beg == index_end) {
@@ -646,7 +662,7 @@ uint64_t  uget_a2cf_completed (UgetA2cf* a2cf)
 	for (index = 0;  index < a2cf->piece.index_end;  index++) {
 		if (test_bit (a2cf->bitfield, index) == TRUE) {
 			if (index == a2cf->piece.index_end - 1) {
-				last_piece_len = a2cf->total_len & (a2cf->piece_len-1);
+				last_piece_len = a2cf->total_len % a2cf->piece_len;
 				if (last_piece_len) {
 					completed += last_piece_len;
 					break;
@@ -704,7 +720,7 @@ UgetA2cfPiece*  uget_a2cf_realloc (UgetA2cf* a2cf, uint32_t piece_index)
 		piece = a2cf_piece_new (a2cf->piece_len);
 		piece->index = piece_index;
 		if (piece_index == a2cf->piece.index_end - 1)
-			a2cf_piece_truncate (piece, a2cf->total_len & (a2cf->piece_len-1));
+			a2cf_piece_truncate (piece, a2cf->total_len % a2cf->piece_len);
 		uget_a2cf_insert (a2cf, piece);
 	}
 	return piece;
