@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -63,56 +63,50 @@
 
 static struct UgetNodeControl  control_real =
 {
-	NULL,                       // struct UgetNodeControl*  children;
-	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
-	{NULL, FALSE},              // struct UgetNodeSort      sort;
-	NULL,                       // UgetNodeFunc             filter;
-	NULL                        // void*                    data;
+//	NULL,                           // struct UgetNodeControl*  children;
+	&uget_node_default_notifier,    // struct UgetNodeNotifier* notifier;
+	{NULL, FALSE},                  // struct UgetNodeSort      sort;
+	NULL,                           // UgetNodeFunc             filter;
 };
 
 static struct UgetNodeControl  control_split =
 {
-	NULL,                       // struct UgetNodeControl*  children;
-	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
-	{NULL, FALSE},              // struct UgetNodeSort      sort;
-	uget_node_filter_split,     // UgetNodeFunc             filter;
-	NULL                        // void*                    data;
+//	NULL,                           // struct UgetNodeControl*  children;
+	&uget_node_default_notifier,    // struct UgetNodeNotifier* notifier;
+	{NULL, FALSE},                  // struct UgetNodeSort      sort;
+	uget_node_filter_split,         // UgetNodeFunc             filter;
 };
 
 static struct UgetNodeControl  control_sorted =
 {
-	NULL,                       // struct UgetNodeControl*  children;
-	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
-	{NULL, FALSE},              // struct UgetNodeSort      sort;
-	uget_node_filter_sorted,    // UgetNodeFunc             filter;
-	NULL                        // void*                    data;
+//	NULL,                           // struct UgetNodeControl*  children;
+	&uget_node_default_notifier,    // struct UgetNodeNotifier* notifier;
+	{NULL, FALSE},                  // struct UgetNodeSort      sort;
+	uget_node_filter_sorted,        // UgetNodeFunc             filter;
 };
 
 static struct UgetNodeControl  control_sorted_split =
 {
-	NULL,                       // struct UgetNodeControl*  children;
-	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
-	{NULL, FALSE},              // struct UgetNodeSort      sort;
-	uget_node_filter_split,     // UgetNodeFunc             filter;
-	NULL                        // void*                    data;
+//	NULL,                           // struct UgetNodeControl*  children;
+	&uget_node_default_notifier,    // struct UgetNodeNotifier* notifier;
+	{NULL, FALSE},                  // struct UgetNodeSort      sort;
+	uget_node_filter_split,         // UgetNodeFunc             filter;
 };
 
 static struct UgetNodeControl  control_mix =
 {
-	NULL,                       // struct UgetNodeControl*  children;
-	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
-	{NULL, FALSE},              // struct UgetNodeSort      sort;
-	uget_node_filter_mix,       // UgetNodeFunc             filter;
-	NULL                        // void*                    data;
+//	NULL,                           // struct UgetNodeControl*  children;
+	&uget_node_default_notifier,    // struct UgetNodeNotifier* notifier;
+	{NULL, FALSE},                  // struct UgetNodeSort      sort;
+	uget_node_filter_mix,           // UgetNodeFunc             filter;
 };
 
 static struct UgetNodeControl  control_mix_split =
 {
-	NULL,                       // struct UgetNodeControl*  children;
-	{NULL, NULL, NULL},         // struct UgetNodeNotifier  notifier;
-	{NULL, FALSE},              // struct UgetNodeSort      sort;
-	uget_node_filter_mix_split, // UgetNodeFunc             filter;
-	NULL                        // void*                    data;
+//	NULL,                           // struct UgetNodeControl*  children;
+	&uget_node_default_notifier,    // struct UgetNodeNotifier* notifier;
+	{NULL, FALSE},                  // struct UgetNodeSort      sort;
+	uget_node_filter_mix_split,     // UgetNodeFunc             filter;
 };
 
 
@@ -136,12 +130,14 @@ void  uget_app_init (UgetApp* app)
 	app->mix_split.control = &control_mix_split;
 	// add virtual category - "All Category"
 	node = uget_node_new (NULL);
-	common = ug_data_realloc(node->data, UgetCommonInfo);
+	common = ug_info_realloc(node->info, UgetCommonInfo);
 	common->name = ug_strdup (_("All Category"));
 	uget_node_append (&app->mix, node);
 
 	uget_task_init (&app->task);
 	ug_array_init (&app->nodes, sizeof (void*), 32);
+	app->uri_hash = NULL;
+	app->config_dir = NULL;
 
 	// plug-in registry
 	app->plugin_default = NULL;
@@ -158,17 +154,23 @@ void  uget_app_init (UgetApp* app)
 	ug_registry_add (&app->infos, UgetRelationInfo);
 	ug_registry_add (&app->infos, UgetCategoryInfo);
 	ug_registry_sort (&app->infos);
-	ug_data_set_registry (&app->infos);
+	ug_info_set_registry (&app->infos);
+	// counter
+	app->n_error = 0;
+	app->n_moved = 0;
+	app->n_deleted = 0;
+	app->n_completed = 0;
 }
 
 void  uget_app_final (UgetApp* app)
 {
 	ug_array_clear (&app->nodes);
 	uget_task_final (&app->task);
-	uget_app_clear_plugins (app);
+	uget_app_clear_plugins (app);    // clear app->plugins
 
 	uget_node_clear_children (&app->mix_split);
 	uget_node_clear_children (&app->mix);
+	uget_node_clear_children (&app->sorted_split);
 	uget_node_clear_children (&app->sorted);
 	uget_node_clear_children (&app->split);
 	uget_node_clear_children (&app->real);
@@ -178,6 +180,8 @@ void  uget_app_final (UgetApp* app)
 
 	uget_uri_hash_free (app->uri_hash);
 	app->uri_hash = NULL;
+	ug_free(app->config_dir);
+	app->config_dir = NULL;
 }
 
 static UgArrayPtr* uget_app_store_nodes (UgetApp* app, UgetNode* parent)
@@ -216,7 +220,7 @@ static int  uget_app_activate (UgetApp* app, UgetNode* cnode, UgetCategory* cate
 	for (index = 0;  index < array->length;  index++) {
 		dnode = array->at[index];
 		uget_node_updated (dnode);
-		relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+		relation = ug_info_realloc(dnode->info, UgetRelationInfo);
 		if (relation->group & UGET_GROUP_ACTIVE) {
 			// remove node and insert it again to sort node
 			if (app->mix.control->sort.compare) {
@@ -236,7 +240,7 @@ static int  uget_app_activate (UgetApp* app, UgetNode* cnode, UgetCategory* cate
 			relation->group |= UGET_GROUP_FINISHED;
 			sibling = category->finished->children;
 			// completed time
-			log = ug_data_realloc (dnode->data, UgetLogInfo);
+			log = ug_info_realloc (dnode->info, UgetLogInfo);
 			log->completed_time = time(NULL);    // get current time
 			app->n_completed++;
 		}
@@ -278,7 +282,7 @@ static void uget_app_queuing (UgetApp* app, UgetNode* cnode, UgetCategory* categ
 		dnode = array->at[index];
 		if (category->active->n_children >= category->active_limit)
 			break;
-		relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+		relation = ug_info_realloc(dnode->info, UgetRelationInfo);
 		if (relation->group & UGET_GROUP_INACTIVE)
 			continue;
 		uget_app_activate_download (app, dnode);
@@ -299,7 +303,7 @@ int  uget_app_grow (UgetApp* app, int no_queuing)
 	uget_task_dispatch (&app->task);
 	// active, queuing, finished, recycled
 	for (cnode = app->real.children;  cnode;  cnode = cnode->next) {
-		category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+		category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 		if (category == NULL)
 			continue;
 		uget_app_activate (app, cnode, category);
@@ -310,7 +314,7 @@ int  uget_app_grow (UgetApp* app, int no_queuing)
 	return n_active;
 }
 
-int   uget_app_trim(UgetApp* app)
+int   uget_app_trim(UgetApp* app, UgArrayPtr* deleted_nodes)
 {
 	UgetCategory*  category;
 	UgetNode*      cnode;
@@ -318,22 +322,28 @@ int   uget_app_trim(UgetApp* app)
 	int            n_deleted_prev = app->n_deleted;
 
 	for (cnode = app->real.children;  cnode;  cnode = cnode->next) {
-		category = ug_data_realloc(cnode->data, UgetCategoryInfo);
+		category = ug_info_realloc(cnode->info, UgetCategoryInfo);
 		if (category == NULL)
 			continue;
 		while (category->finished->n_children > category->finished_limit) {
 			dnode = category->finished->last->real;
-			uget_uri_hash_remove_download(app->uri_hash, dnode->data);
+			uget_uri_hash_remove_download(app->uri_hash, dnode->info);
 			uget_node_remove(cnode, dnode);
 			uget_node_free(dnode);
 			app->n_deleted++;
+			// add deleted nodes to array
+			if (deleted_nodes)
+				*(void**)ug_array_alloc(deleted_nodes, 1) = dnode;
 		}
 		while (category->recycled->n_children > category->recycled_limit) {
 			dnode = category->recycled->last->real;
-			uget_uri_hash_remove_download(app->uri_hash, dnode->data);
+			uget_uri_hash_remove_download(app->uri_hash, dnode->info);
 			uget_node_remove(cnode, dnode);
 			uget_node_free(dnode);
 			app->n_deleted++;
+			// add deleted nodes to array
+			if (deleted_nodes)
+				*(void**)ug_array_alloc(deleted_nodes, 1) = dnode;
 		}
 	}
 	return app->n_deleted - n_deleted_prev;
@@ -415,35 +425,10 @@ void  uget_app_set_notification (UgetApp* app, void* data,
                                  UgetNodeFunc removed,
                                  UgNotifyFunc updated)
 {
-	control_real.notifier.inserted = inserted;
-	control_real.notifier.removed  = removed;
-	control_real.notifier.updated  = updated;
-	control_real.data              = data;
-
-	control_split.notifier.inserted = inserted;
-	control_split.notifier.removed  = removed;
-	control_split.notifier.updated  = updated;
-	control_split.data              = data;
-
-	control_sorted.notifier.inserted = inserted;
-	control_sorted.notifier.removed  = removed;
-	control_sorted.notifier.updated  = updated;
-	control_sorted.data              = data;
-
-	control_sorted_split.notifier.inserted = inserted;
-	control_sorted_split.notifier.removed  = removed;
-	control_sorted_split.notifier.updated  = updated;
-	control_sorted_split.data              = data;
-
-	control_mix.notifier.inserted = inserted;
-	control_mix.notifier.removed  = removed;
-	control_mix.notifier.updated  = updated;
-	control_mix.data              = data;
-
-	control_mix_split.notifier.inserted = inserted;
-	control_mix_split.notifier.removed  = removed;
-	control_mix_split.notifier.updated  = updated;
-	control_mix_split.data              = data;
+	uget_node_default_notifier.inserted = inserted;
+	uget_node_default_notifier.removed  = removed;
+	uget_node_default_notifier.updated  = updated;
+	uget_node_default_notifier.data     = data;
 }
 
 void  uget_app_add_category (UgetApp* app, UgetNode* cnode, int save_file)
@@ -455,7 +440,7 @@ void  uget_app_add_category (UgetApp* app, UgetNode* cnode, int save_file)
 
 	uget_node_append (&app->real, cnode);
 	uget_uri_hash_add_category (app->uri_hash, cnode);
-	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+	category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 	for (node = cnode->fake;  node;  node = node->peer) {
 		switch (uget_node_get_group(node)) {
 		case UGET_GROUP_ACTIVE:
@@ -592,7 +577,7 @@ void  uget_app_stop_category (UgetApp* app, UgetNode* cnode)
 	UgArrayPtr*    array;
 	int            index;
 
-	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+	category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 
 	// Because uget_app_queue_download() will change node linking,
 	// program must store active nodes to array before calling uget_app_queue_download()
@@ -614,7 +599,7 @@ void  uget_app_pause_category (UgetApp* app, UgetNode* cnode)
 	UgArrayPtr*    array;
 	int            index;
 
-	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+	category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 
 	// Because uget_app_queue_download() will change node linking,
 	// program must store active nodes to array before calling uget_app_queue_download()
@@ -648,7 +633,7 @@ void  uget_app_resume_category (UgetApp* app, UgetNode* cnode)
 	UgArrayPtr*    array;
 	int            index;
 
-	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+	category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 
 	// Because uget_app_queue_download() will change node linking,
 	// program must store active nodes to array before calling uget_app_queue_download()
@@ -700,7 +685,7 @@ UgetNode* uget_app_match_category (UgetApp* app, UgUri* uuri, const char* file)
 	matched.count = 0;
 
 	for (cnode = app->real.children;  cnode;  cnode = cnode->next) {
-		category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+		category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 		if (category == NULL)
 			continue;
 		// null-terminated
@@ -738,7 +723,7 @@ int  uget_app_add_download_uri (UgetApp* app, const char* uri, UgetNode* cnode,
 	UgetCommon* common;
 
 	dnode = uget_node_new (NULL);
-	common = ug_data_realloc (dnode->data, UgetCommonInfo);
+	common = ug_info_realloc (dnode->info, UgetCommonInfo);
 	common->uri = ug_strdup (uri);
 	return uget_app_add_download (app, dnode, cnode, apply);
 }
@@ -749,50 +734,64 @@ int  uget_app_add_download (UgetApp* app, UgetNode* dnode, UgetNode* cnode, int
 	UgetRelation* relation_c;
 	UgetNode*     sibling;
 	UgetLog*      log;
-	UgUri         uuri;
+	UgUri*        uuri;
+	char*         fattch;
 	int           value;
 	struct {
 		UgetCommon*   common;
 		UgetCategory* category;
 	} temp;
 
-	temp.common = ug_data_realloc (dnode->data, UgetCommonInfo);
+	temp.common = ug_info_realloc (dnode->info, UgetCommonInfo);
 	// replace invalid characters \/:*?"<>| by _ in filename.
 	if (temp.common->file)
 		ug_str_replace_chars (temp.common->file, "\\/:*?\"<>|", '_');
 	// decode name, filename, and category
 	if (temp.common->uri) {
-		ug_uri_init (&uuri, temp.common->uri);
+		uuri = ug_malloc(sizeof (UgUri));
+		ug_uri_init(uuri, temp.common->uri);
 		// set UgetCommon::name if it's name is NULL
 		if (temp.common->name == NULL) {
 			if (temp.common->file)
 				temp.common->name = ug_strdup(temp.common->file);
 			else
-				temp.common->name = uget_name_from_uri(&uuri);
+				temp.common->name = uget_name_from_uri(uuri);
 		}
+		// backup file
+		if (ug_uri_is_file(uuri)) {
+			fattch = uget_app_save_attachment(app, dnode->info,
+			                                  temp.common->uri + uuri->path, NULL);
+			if (fattch) {
+				// uuri will failure
+				ug_free(temp.common->uri);
+				temp.common->uri = fattch;
+			}
+		}
+		ug_free(uuri);
+		//
 		if (cnode == NULL)
-			cnode = uget_app_match_category (app, &uuri, temp.common->file);
+			cnode = uget_app_match_category (app, uuri, temp.common->file);
 	}
 
 	if (cnode == NULL)
 		cnode = app->real.children;
 	if (cnode) {
-		relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+		relation = ug_info_realloc(dnode->info, UgetRelationInfo);
 		relation->group &= UGET_GROUP_MAJOR | UGET_GROUP_PAUSED;
 		relation->group |= UGET_GROUP_QUEUING;
-		log = ug_data_realloc (dnode->data, UgetLogInfo);
+		log = ug_info_realloc (dnode->info, UgetLogInfo);
 		log->added_time = time (NULL);    // get current time
 		if (apply) {
 			value = temp.common->keeping.enable;
 			temp.common->keeping.enable = TRUE;
 			temp.common->keeping.uri = TRUE;
-			ug_data_assign (dnode->data, cnode->data, UgetCategoryInfo);
+			ug_info_assign (dnode->info, cnode->info, UgetCategoryInfo);
 			temp.common->keeping.enable = value;
-			relation_c = ug_data_realloc(cnode->data, UgetRelationInfo);
+			relation_c = ug_info_realloc(cnode->info, UgetRelationInfo);
 			if (relation_c->group & UGET_GROUP_PAUSED)
 				relation->group |= UGET_GROUP_PAUSED;
 		}
-		temp.category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+		temp.category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 		// try to insert download before finished and recycled
 		sibling = temp.category->finished->children;
 		if (sibling == NULL)
@@ -801,7 +800,7 @@ int  uget_app_add_download (UgetApp* app, UgetNode* dnode, UgetNode* cnode, int
 		if (sibling)
 			sibling = sibling->real;
 		uget_node_insert (cnode, sibling, dnode);
-		uget_uri_hash_add_download(app->uri_hash, dnode->data);
+		uget_uri_hash_add_download(app->uri_hash, dnode->info);
 		return TRUE;
 	}
 	return FALSE;
@@ -832,8 +831,8 @@ int   uget_app_move_download_to (UgetApp* app, UgetNode* dnode, UgetNode* cnode)
 
 	if (dnode->parent == cnode)
 		return FALSE;
-	category = ug_data_realloc(cnode->data, UgetCategoryInfo);
-	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	category = ug_info_realloc(cnode->info, UgetCategoryInfo);
+	relation = ug_info_realloc(dnode->info, UgetRelationInfo);
 
 	switch (relation->group & UGET_GROUP_MAJOR) {
 	case UGET_GROUP_ACTIVE:
@@ -870,10 +869,8 @@ static int  delete_files(UgetFiles* files, int has_temp_file)
 {
 	UgetFile* file1;
 	int  n_error;
-	int  index;
 
-	for (n_error = 0, index = 0;  index < files->collection.length;  index++) {
-		file1 = files->collection.at + index;
+	for (n_error = 0, file1 = (UgetFile*)files->list.head;  file1;  file1 = file1->next) {
 		if (file1->state & UGET_FILE_STATE_DELETED)
 			continue;
 
@@ -902,7 +899,7 @@ static UgThreadResult  delete_files_thread(UgetFiles* files)
 			break;
 	}
 
-	ug_group_data_free(files);
+	ug_data_free(files);
 	return UG_THREAD_RESULT;
 }
 
@@ -916,8 +913,8 @@ int  uget_app_delete_download (UgetApp* app, UgetNode* dnode, int delete_file)
 #ifdef USE__ANDROID__SAF
 	is_active = TRUE;  // delete files in thread if program use Android SAF
 #endif
-	uget_uri_hash_remove_download(app->uri_hash, dnode->data);
-	files = ug_data_set(dnode->data, UgetFilesInfo, NULL);
+	uget_uri_hash_remove_download(app->uri_hash, dnode->info);
+	files = ug_info_set(dnode->info, UgetFilesInfo, NULL);
 	uget_node_free(dnode);
 
 	if (delete_file == TRUE && files) {
@@ -930,7 +927,7 @@ int  uget_app_delete_download (UgetApp* app, UgetNode* dnode, int delete_file)
 	}
 
 	if (files)
-		ug_group_data_free(files);
+		ug_data_free(files);
 	return TRUE;
 }
 
@@ -945,7 +942,7 @@ int  uget_app_recycle_download (UgetApp* app, UgetNode* dnode)
 	uget_task_remove (&app->task, dnode);
 	uget_node_remove (cnode, dnode);
 
-	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	relation = ug_info_realloc(dnode->info, UgetRelationInfo);
 	if (relation->group & UGET_GROUP_RECYCLED) {
 		uget_node_free (dnode);
 		return FALSE;
@@ -954,7 +951,7 @@ int  uget_app_recycle_download (UgetApp* app, UgetNode* dnode)
 		relation->group &= ~UGET_GROUP_MAJOR;
 		relation->group |=  UGET_GROUP_RECYCLED;
 		uget_node_clear_fake (dnode);
-		category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+		category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 		// try to insert download before recycled
 		sibling = category->recycled->children;
 		if (sibling)
@@ -976,14 +973,14 @@ int   uget_app_activate_download (UgetApp* app, UgetNode* dnode)
 		UgetPluginInfo*  pinfo;
 	} temp;
 
-	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	relation = ug_info_realloc(dnode->info, UgetRelationInfo);
 	if (relation->group & UGET_GROUP_ACTIVE)
 		return FALSE;
-	common = ug_data_get (dnode->data, UgetCommonInfo);
+	common = ug_info_get (dnode->info, UgetCommonInfo);
 	if (common == NULL || common->uri == NULL)
 		return FALSE;
 	// match plug-in
-	log = ug_data_realloc (dnode->data, UgetLogInfo);
+	log = ug_info_realloc (dnode->info, UgetLogInfo);
 	temp.pinfo = uget_app_match_plugin (app, common->uri, NULL);
 	if (temp.pinfo == NULL) {
 		// no plug-in support
@@ -1016,7 +1013,7 @@ int   uget_app_activate_download (UgetApp* app, UgetNode* dnode)
 	uget_node_remove (cnode, dnode);
 	uget_node_clear_fake (dnode);
 	relation->group =  UGET_GROUP_ACTIVE;
-	temp.category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+	temp.category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 	// try to insert download before queuing, finished, and recycled
 	sibling = temp.category->queuing->children;
 	if (sibling == NULL)
@@ -1039,14 +1036,14 @@ int   uget_app_pause_download (UgetApp* app, UgetNode* dnode)
 	UgetNode*     cnode;
 	UgetNode*     fake;
 
-	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	relation = ug_info_realloc(dnode->info, UgetRelationInfo);
 	if (relation->group & UGET_GROUP_UNRUNNABLE)
 		return FALSE;
 	uget_task_remove (&app->task, dnode);
 	relation->group |= UGET_GROUP_PAUSED;
 
 	cnode = dnode->parent;
-	category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+	category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 	for (fake = dnode->fake;  fake;  fake = fake->peer) {
 		if (fake->parent == category->active) {
 			uget_node_remove (cnode, dnode);
@@ -1069,11 +1066,11 @@ int   uget_app_pause_download (UgetApp* app, UgetNode* dnode)
 #else
 	UgetRelation* relation;
 
-	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	relation = ug_info_realloc(dnode->info, UgetRelationInfo);
 	if (relation->group & UGET_GROUP_ACTIVE) {
 //		uget_task_remove (&app->task, dnode);
-		if (relation && relation->task.plugin)
-			uget_plugin_stop (relation->task.plugin);
+		if (relation && relation->task)
+			uget_plugin_stop (relation->task->plugin);
 		relation->group &= ~UGET_GROUP_ACTIVE;
 	}
 	else if (relation->group & UGET_GROUP_UNRUNNABLE)
@@ -1090,7 +1087,7 @@ int   uget_app_queue_download (UgetApp* app, UgetNode* dnode)
 	UgetCategory* category;
 	UgetRelation* relation;
 
-	relation = ug_data_realloc(dnode->data, UgetRelationInfo);
+	relation = ug_info_realloc(dnode->info, UgetRelationInfo);
 	if ((relation->group & UGET_GROUP_ACTIVE)     == 0 &&
 		(relation->group & UGET_GROUP_UNRUNNABLE) == 0)
 		return FALSE;
@@ -1105,7 +1102,7 @@ int   uget_app_queue_download (UgetApp* app, UgetNode* dnode)
 		// --- decide sibling position & insert before it ---
 		// if current download is in active, insert it before queuing,
 		// otherwise insert it before finished and recycled.
-		category = ug_data_realloc (cnode->data, UgetCategoryInfo);
+		category = ug_info_realloc (cnode->info, UgetCategoryInfo);
 		sibling = NULL;
 		if (relation->group & UGET_GROUP_ACTIVE) {
 			uget_task_remove (&app->task, dnode);
@@ -1130,7 +1127,7 @@ void  uget_app_reset_download_name (UgetApp* app, UgetNode* dnode)
 	UgetNode*    sibling;
 	UgetNode*    cnode = NULL;
 
-	common = ug_data_realloc(dnode->data, UgetCommonInfo);
+	common = ug_info_realloc(dnode->info, UgetCommonInfo);
 	if (common->file) {
 		if (common->name && strcmp(common->file, common->name) == 0)
 			return;
@@ -1166,10 +1163,112 @@ void  uget_app_use_uri_hash (UgetApp* app)
 		uget_uri_hash_add_category (app->uri_hash, cnode);
 }
 
+char* uget_app_save_attachment(UgetApp* app, UgInfo* info,
+                               const char* src_path, const char* rename)
+{
+	char*       dest_path;
+	int         count;
+	UgetFile*   file1;
+	UgetFiles*  files;
+
+	if (rename == NULL) {
+		rename = strrchr(src_path, UG_DIR_SEPARATOR);
+#if defined _WIN32 || defined _WIN64
+		if (rename == NULL)
+			rename = strrchr(src_path, '/');
+#endif
+		if (rename == NULL)
+			rename = src_path;
+		else
+			rename++;
+	}
+
+	dest_path = ug_strdup_printf(
+			"%s" UG_DIR_SEPARATOR_S "attachment" UG_DIR_SEPARATOR_S "%s",
+			app->config_dir, rename);
+
+	for (count = 0;  ug_file_is_exist(dest_path);  count++) {
+		ug_free(dest_path);
+		dest_path = ug_strdup_printf(
+				"%s" UG_DIR_SEPARATOR_S "attachment" UG_DIR_SEPARATOR_S "%d) %s",
+				app->config_dir, count, rename);
+	}
+
+	if (ug_file_copy(src_path, dest_path) == -1) {
+		ug_free(dest_path);
+		return NULL;
+	}
+	// add new path to UgetFiles
+	files = ug_info_realloc(info, UgetFilesInfo);
+	file1 = uget_files_realloc(files, dest_path);
+	file1->type = UGET_FILE_ATTACHMENT;
+
+	return dest_path;
+}
+
+/*
+void  uget_app_store_attachment(UgetApp* app, UgInfo* info)
+{
+	int    num;
+	char*  fname;
+	char*  fattch;
+	UgUri* uuri;
+	union {
+		UgetHttp*    http;
+		UgetCommon*  common;
+	} tmp;
+
+	tmp.common = ug_info_get(info, UgetCommonInfo);
+	if (tmp.common && tmp.common->uri) {
+		uuri = ug_malloc(sizeof(UgUri));
+		ug_uri_init(uuri, tmp.common->uri);
+		if (ug_uri_is_file(uuri)) {
+			fattch = uget_app_save_attachment(app, info,
+			                                  tmp.common->uri + uuri->path, NULL);
+			if (fattch) {
+				ug_free(tmp.common->uri);
+				tmp.common->uri = fattch;
+			}
+		}
+		ug_free(uuri);
+	}
+
+	srand((unsigned int) time(NULL));
+	num = rand();
+
+	tmp.http = ug_info_get(info, UgetHttpInfo);
+	// backup cookie file
+	if (tmp.http && tmp.http->cookie_file) {
+		fname = ug_strdup_printf("%X.cookie", num);
+		fattch = uget_app_save_attachment(app, info,
+		                                  tmp.http->cookie_file, fname);
+		ug_free(fname);
+		if (fattch) {
+			ug_free(tmp.http->cookie_file);
+			tmp.http->cookie_file = fattch;
+		}
+	}
+
+	// backup post file
+	if (tmp.http && tmp.http->post_file) {
+		fname = ug_strdup_printf("%X.post", num);
+		fattch = uget_app_save_attachment(app, info,
+		                                  tmp.http->post_file, fname);
+		ug_free(fname);
+		if (fattch) {
+			ug_free(tmp.http->post_file);
+			tmp.http->post_file = fattch;
+		}
+	}
+}
+ */
+
 void  uget_app_clear_attachment (UgetApp* app)
 {
 	UgetNode*   dnode;
 	UgetHttp*   http;
+	UgetFiles*  files;
+	UgetFile*   file1;
 	UgDir*      dir;
 	void*       hash;
 	const char* name;
@@ -1179,12 +1278,20 @@ void  uget_app_clear_attachment (UgetApp* app)
 	hash = uget_uri_hash_new ();
 	// add attachment
 	for (dnode = app->mix.children->children;  dnode;  dnode = dnode->next) {
-		if ((http = ug_data_get (dnode->data, UgetHttpInfo)) == NULL)
-			continue;
-		if (http->cookie_file)
-			uget_uri_hash_add (hash, http->cookie_file);
-		if (http->post_file)
-			uget_uri_hash_add (hash, http->post_file);
+		// UgetFiles
+		if ((files = ug_info_get(dnode->info, UgetFilesInfo)) != NULL ) {
+			for (file1 = (UgetFile*)files->list.head;  file1;  file1 = file1->next) {
+				if (file1->type == UGET_FILE_ATTACHMENT)
+					uget_uri_hash_add(hash, file1->path);
+			}
+		}
+		// UgetHttp
+		if ((http = ug_info_get(dnode->info, UgetHttpInfo)) != NULL) {
+			if (http->cookie_file)
+				uget_uri_hash_add(hash, http->cookie_file);
+			if (http->post_file)
+				uget_uri_hash_add(hash, http->post_file);
+		}
 	}
 
 	folder = ug_build_filename (app->config_dir, "attachment", NULL);
@@ -1532,7 +1639,7 @@ void  uget_node_set_keeping (UgetNode* node, int enable)
 		UgetFtp*     ftp;
 	} temp;
 
-	temp.common = ug_data_realloc (node->data, UgetCommonInfo);
+	temp.common = ug_info_realloc (node->info, UgetCommonInfo);
 	if (temp.common) {
 		temp.common->keeping.enable = enable;
 		if (enable) {
@@ -1570,7 +1677,7 @@ void  uget_node_set_keeping (UgetNode* node, int enable)
 		}
 	}
 
-	temp.proxy = ug_data_get (node->data, UgetProxyInfo);
+	temp.proxy = ug_info_get (node->info, UgetProxyInfo);
 	if (temp.proxy) {
 		temp.proxy->keeping.enable = enable;
 		if (enable) {
@@ -1587,7 +1694,7 @@ void  uget_node_set_keeping (UgetNode* node, int enable)
 		}
 	}
 
-	temp.http = ug_data_get (node->data, UgetHttpInfo);
+	temp.http = ug_info_get (node->info, UgetHttpInfo);
 	if (temp.http) {
 		temp.http->keeping.enable = enable;
 		if (enable) {
@@ -1612,7 +1719,7 @@ void  uget_node_set_keeping (UgetNode* node, int enable)
 		}
 	}
 
-	temp.ftp = ug_data_get (node->data, UgetFtpInfo);
+	temp.ftp = ug_info_get (node->info, UgetFtpInfo);
 	if (temp.ftp) {
 		temp.ftp->keeping.enable = enable;
 		if (enable) {
