@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -39,7 +39,7 @@
 
 #include <stdint.h>
 #include <UgUri.h>
-#include <UgGroupData.h>
+#include <UgData.h>
 #include <UgetFiles.h>
 #include <UgetPlugin.h>
 
@@ -57,29 +57,29 @@ typedef struct  UgetLog         UgetLog;
 typedef struct  UgetRelation    UgetRelation;
 typedef struct  UgetCategory    UgetCategory;
 
-extern const UgGroupDataInfo*   UgetCommonInfo;
-extern const UgGroupDataInfo*   UgetProgressInfo;
-extern const UgGroupDataInfo*   UgetProxyInfo;
-extern const UgGroupDataInfo*   UgetHttpInfo;
-extern const UgGroupDataInfo*   UgetFtpInfo;
-extern const UgGroupDataInfo*   UgetLogInfo;
-extern const UgGroupDataInfo*   UgetRelationInfo;
-extern const UgGroupDataInfo*   UgetCategoryInfo;
+extern const UgDataInfo*  UgetCommonInfo;
+extern const UgDataInfo*  UgetProgressInfo;
+extern const UgDataInfo*  UgetProxyInfo;
+extern const UgDataInfo*  UgetHttpInfo;
+extern const UgDataInfo*  UgetFtpInfo;
+extern const UgDataInfo*  UgetLogInfo;
+extern const UgDataInfo*  UgetRelationInfo;
+extern const UgDataInfo*  UgetCategoryInfo;
 
 /* ----------------------------------------------------------------------------
-   UgetCommon: It derived from UgGroupData and store in UgData.
+   UgetCommon: It derived from UgData and store in UgInfo.
 
    UgType
    |
-   `-- UgGroupData
+   `-- UgData
        |
        `-- UgetCommon
  */
 
 struct UgetCommon
 {
-	UG_GROUP_DATA_MEMBERS;
-//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
+	UG_DATA_MEMBERS;
+//	const UgDataInfo*  info;    // UgData(UgType) member
 
 	char*   name;
 	char*   uri;
@@ -106,7 +106,7 @@ struct UgetCommon
 	// debug
 	int           debug_level;
 
-	// keeping flags used by ug_group_data_assign ()
+	// keeping flags used by ug_data_assign ()
 	// They works like read-only
 	struct {
 		uint8_t   enable:1;
@@ -138,19 +138,19 @@ char* uget_name_from_uri(UgUri* uri);
 char* uget_name_from_uri_str(const char* uri);
 
 /* ----------------------------------------------------------------------------
-   UgetProgress: It derived from UgGroupData and store in UgData.
+   UgetProgress: It derived from UgData and store in UgInfo.
 
    UgType
    |
-   `-- UgGroupData
+   `-- UgData
        |
        `-- UgetProgress
  */
 
 struct UgetProgress
 {
-	UG_GROUP_DATA_MEMBERS;
-//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
+	UG_DATA_MEMBERS;
+//	const UgDataInfo*  info;  // UgData(UgType) member
 
 	int64_t      elapsed;     // consume time (seconds)
 	int64_t      left;        // remain time  (seconds)
@@ -166,11 +166,11 @@ struct UgetProgress
 };
 
 /* ----------------------------------------------------------------------------
-   UgetProxy: It derived from UgGroupData and store in UgData.
+   UgetProxy: It derived from UgData and store in UgInfo.
 
    UgType
    |
-   `-- UgGroupData
+   `-- UgData
        |
        `-- UgetProxy
  */
@@ -191,8 +191,8 @@ typedef enum
 
 struct UgetProxy
 {
-	UG_GROUP_DATA_MEMBERS;
-//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
+	UG_DATA_MEMBERS;
+//	const UgDataInfo*  info;    // UgData(UgType) member
 
 	char*          host;
 	unsigned int   port;
@@ -230,19 +230,19 @@ struct UgetProxy
 };
 
 /* ----------------------------------------------------------------------------
-   UgetHttp: It derived from UgGroupData and store in UgData.
+   UgetHttp: It derived from UgData and store in UgInfo.
 
    UgType
    |
-   `-- UgGroupData
+   `-- UgData
        |
        `-- UgetHttp
  */
 
 struct UgetHttp
 {
-	UG_GROUP_DATA_MEMBERS;
-//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
+	UG_DATA_MEMBERS;
+//	const UgDataInfo*  info;    // UgData(UgType) member
 
 	char*  user;
 	char*  password;
@@ -274,19 +274,19 @@ struct UgetHttp
 };
 
 /* ----------------------------------------------------------------------------
-   UgetFtp: It derived from UgGroupData and store in UgData.
+   UgetFtp: It derived from UgData and store in UgInfo.
 
    UgType
    |
-   `-- UgGroupData
+   `-- UgData
        |
        `-- UgetFtp
  */
 
 struct UgetFtp
 {
-	UG_GROUP_DATA_MEMBERS;
-//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
+	UG_DATA_MEMBERS;
+//	const UgDataInfo*  info;    // UgData(UgType) member
 
 	char*		 user;
 	char*		 password;
@@ -307,15 +307,15 @@ struct UgetFtp
 
    UgType
    |
-   `-- UgGroupData
+   `-- UgData
        |
        `-- UgetLog
  */
 
 struct UgetLog
 {
-	UG_GROUP_DATA_MEMBERS;
-//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
+	UG_DATA_MEMBERS;
+//	const UgDataInfo*  info;    // UgData(UgType) member
 
 	time_t  added_time;
 	time_t  completed_time;
@@ -324,11 +324,11 @@ struct UgetLog
 };
 
 /* ----------------------------------------------------------------------------
-   UgetRelation: It derived from UgGroupData and store in UgData.
+   UgetRelation: It derived from UgData and store in UgInfo.
 
    UgType
    |
-   `-- UgGroupData
+   `-- UgData
        |
        `-- UgetRelation
  */
@@ -342,51 +342,36 @@ typedef enum
 
 struct UgetRelation
 {
-	UG_GROUP_DATA_MEMBERS;
-//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
+	UG_DATA_MEMBERS;
+//	const UgDataInfo*  info;    // UgData(UgType) member
 
-	int   group;     // UgetGroup
+	int    group;      // UgetGroup
+	int    priority;   // UgetPriority
 
 	// used by UgetTask
 	struct UgetRelationTask {
 		UgetRelation*  prev;
 		UgetPlugin*  plugin;
-		char*        plugin_name;
-		int          priority;   // UgetPriority
 		// speed control
 		int          speed[2];   // current speed
 		int          limit[2];   // current speed limit
-	} task;
-
-	// call destroy.func(destroy.data) when destroying.
-	struct {
-		UgNotifyFunc func;
-		void*        data;
-	} destroy;
-
-	// used by user application
-	struct {
-		void*  pointer;
-		void*  storage;
-		void*  position;
-		void*  data[4];
-	} user;
+	}* task;
 };
 
 /* ----------------------------------------------------------------------------
-   UgetCategory: It derived from UgGroupData and store in UgData.
+   UgetCategory: It derived from UgData and store in UgInfo.
 
    UgType
    |
-   `-- UgGroupData
+   `-- UgData
        |
        `-- UgetCategory
  */
 
 struct UgetCategory
 {
-	UG_GROUP_DATA_MEMBERS;
-//	const UgGroupDataInfo*  info;    // UgGroupData(UgType) member
+	UG_DATA_MEMBERS;
+//	const UgDataInfo*  info;    // UgData(UgType) member
 
 	// use these to classify download
 	UgArrayStr    hosts;
@@ -418,15 +403,63 @@ struct UgetCategory
 namespace Uget
 {
 
+const UgDataInfo* const CommonInfo   = UgetCommonInfo;
+const UgDataInfo* const ProgressInfo = UgetProgressInfo;
+const UgDataInfo* const ProxyInfo    = UgetProxyInfo;
+const UgDataInfo* const HttpInfo     = UgetHttpInfo;
+const UgDataInfo* const FtpInfo      = UgetFtpInfo;
+const UgDataInfo* const LogInfo      = UgetLogInfo;
+const UgDataInfo* const RelationInfo = UgetRelationInfo;
+const UgDataInfo* const CategoryInfo = UgetCategoryInfo;
+
 // These are for directly use only. You can NOT derived it.
-struct Common : Ug::GroupDataMethod, UgetCommon {};
-struct Progress : Ug::GroupDataMethod, UgetProgress {};
-struct Proxy : Ug::GroupDataMethod, UgetProxy {};
-struct Http : Ug::GroupDataMethod, UgetHttp {};
-struct Ftp : Ug::GroupDataMethod, UgetFtp {};
-struct Log : Ug::GroupDataMethod, UgetLog {};
-struct Relation : Ug::GroupDataMethod, UgetRelation {};
-struct Category : Ug::GroupDataMethod, UgetCategory {};
+struct Common   : Ug::DataMethod<Common>, UgetCommon
+{
+	inline void* operator new(size_t size)
+		{ return ug_data_new(UgetCommonInfo); }
+};
+
+struct Progress : Ug::DataMethod<Progress>, UgetProgress
+{
+	inline void* operator new(size_t size)
+		{ return ug_data_new(UgetProgressInfo); }
+};
+
+struct Proxy    : Ug::DataMethod<Proxy>, UgetProxy
+{
+	inline void* operator new(size_t size)
+		{ return ug_data_new(UgetProxyInfo); }
+};
+
+struct Http     : Ug::DataMethod<Http>, UgetHttp
+{
+	inline void* operator new(size_t size)
+		{ return ug_data_new(UgetHttpInfo); }
+};
+
+struct Ftp      : Ug::DataMethod<Ftp>, UgetFtp
+{
+	inline void* operator new(size_t size)
+		{ return ug_data_new(UgetFtpInfo); }
+};
+
+struct Log      : Ug::DataMethod<Log>, UgetLog
+{
+	inline void* operator new(size_t size)
+		{ return ug_data_new(UgetLogInfo); }
+};
+
+struct Relation : Ug::DataMethod<Relation>, UgetRelation
+{
+	inline void* operator new(size_t size)
+		{ return ug_data_new(UgetRelationInfo); }
+};
+
+struct Category : Ug::DataMethod<Category>, UgetCategory
+{
+	inline void* operator new(size_t size)
+		{ return ug_data_new(UgetCategoryInfo); }
+};
 
 };  // namespace Uget
 
