@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2018 by C.H. Huang
+ *   Copyright (C) 2005-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -37,7 +37,7 @@
 #ifndef UGET_OPTION_H
 #define UGET_OPTION_H
 
-#include <UgData.h>
+#include <UgInfo.h>
 #include <UgList.h>
 #include <UgOption.h>
 
@@ -101,7 +101,7 @@ void  uget_option_value_init (UgetOptionValue* value);
 void  uget_option_value_clear (UgetOptionValue* value);
 
 int   uget_option_value_has_ctrl (UgetOptionValue* value);
-int   uget_option_value_to_data (UgetOptionValue* ivalue, UgData* data);
+int   uget_option_value_to_info (UgetOptionValue* ivalue, UgInfo* info);
 
 #ifdef __cplusplus
 }
