@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -39,8 +39,8 @@
 
 #include <stdint.h>
 #include <UgUri.h>
-#include <UgGroupData.h>
 #include <UgData.h>
+#include <UgInfo.h>
 #include <UgThread.h>
 #include <UgetEvent.h>
 
@@ -83,7 +83,7 @@ typedef enum {
 } UgetResult;
 
 // accept/sync return TRUE or FALSE
-typedef int        (*UgetPluginSyncFunc)(UgetPlugin* plugin, UgData* data);
+typedef int        (*UgetPluginSyncFunc)(UgetPlugin* plugin, UgInfo* info);
 // start/stop...etc return TRUE or FALSE.
 typedef int        (*UgetPluginCtrlFunc)(UgetPlugin* plugin, int, void* data);
 // global_set/global_get
@@ -114,7 +114,7 @@ struct UgetPluginInfo
 	// ----------------------------
 	// Global data and functions
 
-	// global data is used for matching UgetPlugin and UgData
+	// global data is used for matching UgetPlugin and UgInfo
 	const char**    hosts;
 	const char**    schemes;
 	const char**    file_exts;
@@ -141,24 +141,24 @@ int     uget_plugin_match(const UgetPluginInfo* info, UgUri* uuri);
    |
    `-- UgetPlugin
 
-                 accept(data)                 accept(data)
+                 accept(info)                 accept(info)
   ,----------. -------------> ,-----------. -------------> ,-----------.
   |          |                |           |                |           |
   | User App |                | plug-in 1 |                | plug-in 2 |
   |          |                |           |                |           |
   `----------' <------------> `-----------' <------------> `-----------'
-               sync(data)                   sync(data)
+               sync(info)                   sync(info)
 
 	// create and start plug-in
 	plugin = uget_plugin_new(UgetPluginCurlInfo);
-	uget_plugin_accept(plugin, data);
+	uget_plugin_accept(plugin, info);
 	if (uget_plugin_start(plugin) == FALSE) {
 		uget_plugin_unref(plugin);
 		return;
 	}
 
 	// Running loop sample 1: use uget_plugin_sync()
-	while (uget_plugin_sync(plugin, data)) {
+	while (uget_plugin_sync(plugin, info)) {
 		// sleep();
 		// do something here
 	}
@@ -169,7 +169,7 @@ int     uget_plugin_match(const UgetPluginInfo* info, UgUri* uuri);
 		// do something here
 		// If you don't want to exchange data (e.g. progress) with plug-in,
 		// you do not need to call uget_plugin_sync() at last.
-		uget_plugin_sync(plugin, data);
+		uget_plugin_sync(plugin, info);
 	} while (uget_plugin_get_state(plugin));
  */
 
@@ -198,13 +198,13 @@ UgetPlugin* uget_plugin_new(const UgetPluginInfo* info);
 void    uget_plugin_ref(UgetPlugin* plugin);
 void    uget_plugin_unref(UgetPlugin* plugin);
 
-// return TRUE  if UgData was accepted by plug-in.
-// return FALSE if UgData lacks necessary data.
-int     uget_plugin_accept(UgetPlugin* plugin, UgData* data);
+// return TRUE  if UgInfo was accepted by plug-in.
+// return FALSE if UgInfo lacks necessary data.
+int     uget_plugin_accept(UgetPlugin* plugin, UgInfo* info);
 
 // return TRUE  if plug-in is running or some data need to exchange/sync.
 // return FALSE if plug-in was stopped and no data need to exchange/sync.
-int     uget_plugin_sync(UgetPlugin* plugin, UgData* data);
+int     uget_plugin_sync(UgetPlugin* plugin, UgInfo* info);
 
 // return TRUE or FALSE.
 int     uget_plugin_ctrl(UgetPlugin* plugin, int code, void* data);
@@ -252,45 +252,53 @@ struct PluginInfoMethod
 };
 
 // This one is for directly use only. You can NOT derived it.
-struct PluginInfo : Uget::PluginInfoMethod, UgetPluginInfo {};
+struct PluginInfo : PluginInfoMethod, UgetPluginInfo {};
 
 
 // This one is for derived use only. No data members here.
 // Your derived struct/class must be C++11 standard-layout
 struct PluginMethod
 {
-	// static method
-	static inline UgetPlugin* create(UgetPluginInfo* pinfo)
+	inline void* operator new(size_t size, const UgetPluginInfo* pinfo)
 		{ return uget_plugin_new(pinfo); }
+	inline void  operator delete(void* p)
+		{ uget_plugin_unref((UgetPlugin*)p); }
 
 	inline void  ref(void)
-		{ uget_plugin_ref((UgetPlugin*) this); }
+		{ uget_plugin_ref((UgetPlugin*)this); }
 	inline void  unref(void)
-		{ uget_plugin_unref((UgetPlugin*) this); }
+		{ uget_plugin_unref((UgetPlugin*)this); }
 
-	inline int   accept(UgData* data)
-		{ return uget_plugin_accept((UgetPlugin*) this, data); }
+	inline int   accept(UgInfo* info)
+		{ return uget_plugin_accept((UgetPlugin*)this, info); }
+	inline int   sync(UgInfo* info)
+		{ return uget_plugin_sync((UgetPlugin*)this, info); }
 
-	inline int   sync(UgData* data)
-		{ return uget_plugin_sync((UgetPlugin*) this, data); }
+	inline int   ctrl(int code, void* data)
+		{ return uget_plugin_ctrl((UgetPlugin*)this, code, data); }
+	inline int   ctrlSpeed(int* DL_UL_array)
+		{ return uget_plugin_ctrl((UgetPlugin*)this, UGET_PLUGIN_CTRL_SPEED, DL_UL_array); }
+	inline int   ctrlSpeed(int dlspeed, int ulspeed) {
+		int  DL_UL_array[2] = {dlspeed, ulspeed};
+		return uget_plugin_ctrl((UgetPlugin*)this, UGET_PLUGIN_CTRL_SPEED, DL_UL_array);
+	}
 
-	inline void  start(UgData* data)
-		{ uget_plugin_start((UgetPlugin*) this); }
-
-	inline void  stop(void)
-		{ uget_plugin_stop((UgetPlugin*) this); }
+	inline int   start(void)
+		{ return uget_plugin_start((UgetPlugin*)this); }
+	inline int   stop(void)
+		{ return uget_plugin_stop((UgetPlugin*)this); }
 
 	inline int   getState(void)
-		{ return uget_plugin_get_state((UgetPlugin*) this); }
+		{ return uget_plugin_get_state((UgetPlugin*)this); }
 
 	inline void  post(UgetEvent* message)
-		{ uget_plugin_post((UgetPlugin*) this, message); }
+		{ uget_plugin_post((UgetPlugin*)this, message); }
 	inline UgetEvent* pop(void)
-		{ return uget_plugin_pop((UgetPlugin*) this); }
+		{ return uget_plugin_pop((UgetPlugin*)this); }
 };
 
 // This one is for directly use only. You can NOT derived it.
-struct Plugin : Uget::PluginMethod, UgetPlugin {};
+struct Plugin : PluginMethod, UgetPlugin {};
 
 };  // namespace Uget
 
