@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2011-2018 by C.H. Huang
+ *   Copyright (C) 2011-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -50,7 +50,7 @@
 
 #ifdef HAVE_LIBPWMD
 #include "pwmd.h"
-static gboolean	uget_plugin_aria2_set_proxy_pwmd (UgetPluginAria2 *plugin, UgData* data, UgValue* options);
+static gboolean	uget_plugin_aria2_set_proxy_pwmd (UgetPluginAria2 *plugin, UgInfo* info, UgValue* options);
 #endif
 
 #if defined _WIN32 || defined _WIN64
@@ -100,13 +100,13 @@ typedef enum Aria2Status {
 } Aria2Status;
 
 // ----------------------------------------------------------------------------
-// UgetPluginInfo (derived from UgGroupDataInfo)
+// UgetPluginInfo (derived from UgTypeInfo)
 
 static void plugin_init (UgetPluginAria2* plugin);
 static void plugin_final(UgetPluginAria2* plugin);
 static int  plugin_ctrl (UgetPluginAria2* plugin, int code, void* data);
-static int  plugin_accept(UgetPluginAria2* plugin, UgData* data);
-static int  plugin_sync  (UgetPluginAria2* plugin, UgData* data);
+static int  plugin_accept(UgetPluginAria2* plugin, UgInfo* node_info);
+static int  plugin_sync  (UgetPluginAria2* plugin, UgInfo* node_info);
 static UgetResult  global_set(int code, void* parameter);
 static UgetResult  global_get(int code, void* parameter);
 
@@ -330,7 +330,7 @@ static void  plugin_final(UgetPluginAria2* plugin)
 	ug_array_clear(&plugin->gids);
 	// clear UgetFiles
 	if (plugin->files)
-		ug_group_data_free(plugin->files);
+		ug_data_free(plugin->files);
 	// clear and recycle start_request object
 	if (plugin->start_request) {
 		ug_value_foreach(&plugin->start_request->params, ug_value_set_name, NULL);
@@ -407,11 +407,12 @@ static int  plugin_ctrl_speed(UgetPluginAria2* plugin, int* speed)
 // plugin_sync
 
 // return FALSE if plug-in was stopped.
-static int  plugin_sync(UgetPluginAria2* plugin, UgData* data)
+static int  plugin_sync(UgetPluginAria2* plugin, UgInfo* node_info)
 {
 	int          index;
 	UgetEvent*   event;
 	UgetFiles*   files;
+	UgetFile*    file1;
 	struct {
 		UgetCommon*      common;
 		UgetProgress*    progress;
@@ -430,7 +431,7 @@ static int  plugin_sync(UgetPluginAria2* plugin, UgData* data)
 	// sync data between plug-in and foreign UgData
 	// ------------------------------------------------
 	// update progress
-	temp.progress = ug_data_realloc(data, UgetProgressInfo);
+	temp.progress = ug_info_realloc(node_info, UgetProgressInfo);
 	temp.progress->complete       = plugin->completedLength;
 	temp.progress->total          = plugin->totalLength;
 	temp.progress->download_speed = plugin->downloadSpeed;
@@ -451,31 +452,33 @@ static int  plugin_sync(UgetPluginAria2* plugin, UgData* data)
 	if (temp.progress->download_speed > 0 && temp.progress->total > 0)
 		temp.progress->left = (temp.progress->total - temp.progress->complete) / temp.progress->download_speed;
 
-	temp.common = ug_data_realloc(data, UgetCommonInfo);
+	temp.common = ug_info_realloc(node_info, UgetCommonInfo);
 	// ------------------------------------------------
-	// sync changed limit from foreign UgData
+	// sync changed limit from foreign UgInfo
 	if (plugin->limit_upper[1] != temp.common->max_upload_speed ||
 		plugin->limit_upper[0] != temp.common->max_download_speed)
 	{
+		// speed control
 		plugin->limit_upper[1] = temp.common->max_upload_speed;
 		plugin->limit_upper[0] = temp.common->max_download_speed;
 		plugin_ctrl_speed(plugin, plugin->limit_upper);
 	}
 
 	// update UgetFiles
-	files = ug_data_realloc(data, UgetFilesInfo);
+	files = ug_info_realloc(node_info, UgetFilesInfo);
 	uget_plugin_lock(plugin);
 	uget_files_sync(files, plugin->files);
 	uget_plugin_unlock(plugin);
 
 	// change name.
-	if (files->collection.length > 0 &&
+	if (files->list.size > 0 &&
 	    plugin->named == FALSE && plugin->files_per_gid > 0)
 	{
 		plugin->named  = TRUE;
 		if (plugin->uri_type == URI_NET && temp.common->file == NULL) {
 			uget_plugin_lock(plugin);
-			ug_uri_init(&plugin->uri_part, files->collection.at[0].path);
+			file1 = (UgetFile*) plugin->files->list.head;
+			ug_uri_init(&plugin->uri_part, file1->path);
 			uget_plugin_unlock(plugin);
 			index = plugin->uri_part.file;
 			if (index != -1) {
@@ -527,7 +530,8 @@ static int  plugin_sync(UgetPluginAria2* plugin, UgData* data)
 				ug_free(temp.common->uri);
 				ug_free(temp.common->name);
 				ug_free(temp.common->file);
-				temp.common->uri  = ug_strdup(plugin->files->collection.at[0].path);
+				file1 = (UgetFile*) plugin->files->list.head;
+				temp.common->uri  = ug_strdup(file1->path);
 				temp.common->name = uget_name_from_uri_str(temp.common->uri);
 				temp.common->file = NULL;
 #ifndef NDEBUG
@@ -896,7 +900,7 @@ exit:
 // ----------------------------------------------------------------------------
 // plugin_accept/plugin_start
 
-static int  plugin_accept(UgetPluginAria2* plugin, UgData* data)
+static int  plugin_accept(UgetPluginAria2* plugin, UgInfo* node_info)
 {
 	UgJsonrpcObject*  request;
 	UgValue*  value;
@@ -913,7 +917,7 @@ static int  plugin_accept(UgetPluginAria2* plugin, UgData* data)
 		UgetHttp*    ftp;
 	} temp;
 
-	temp.common = ug_data_get(data, UgetCommonInfo);
+	temp.common = ug_info_get(node_info, UgetCommonInfo);
 	if (temp.common == NULL || temp.common->uri == NULL)
 		return FALSE;
 
@@ -946,10 +950,6 @@ static int  plugin_accept(UgetPluginAria2* plugin, UgData* data)
 	else if (plugin->uri_part.scheme_len == 6 && strncmp(uri, "magnet", 6) == 0)
 		plugin->uri_type = URI_MAGNET;
 
-	// speed limit control
-	plugin->limit_upper[1] = temp.common->max_upload_speed;
-	plugin->limit_upper[0] = temp.common->max_download_speed;
-
 	request = uget_aria2_alloc(global.data, TRUE, TRUE);
 	if (request->params.type == UG_VALUE_NONE)
 		ug_value_init_array(&request->params, 3);
@@ -1035,6 +1035,11 @@ static int  plugin_accept(UgetPluginAria2* plugin, UgData* data)
 	member->type = UG_VALUE_STRING;
 	member->c.string = ug_strdup_printf("%u", temp.common->retry_limit);
 
+	// speed control: decide speed limit before starting plug-in
+	plugin->limit_upper[1] = temp.common->max_upload_speed;
+	plugin->limit_upper[0] = temp.common->max_download_speed;
+	plugin_ctrl_speed(plugin, plugin->limit);
+
 	member = ug_value_alloc(value, 1);
 	member->name = "max-download-limit";
 	member->type = UG_VALUE_STRING;
@@ -1069,16 +1074,16 @@ static int  plugin_accept(UgetPluginAria2* plugin, UgData* data)
 				temp.common->max_connections);
 	}
 
-	temp.files = ug_data_get(data, UgetFilesInfo);
+	temp.files = ug_info_get(node_info, UgetFilesInfo);
 	if (temp.files)
-		plugin->files = ug_group_data_copy(temp.files);
+		plugin->files = ug_data_copy(temp.files);
 	else
-		plugin->files = ug_group_data_new(UgetFilesInfo);
+		plugin->files = ug_data_new(UgetFilesInfo);
 
-	temp.proxy = ug_data_get(data, UgetProxyInfo);
+	temp.proxy = ug_info_get(node_info, UgetProxyInfo);
 #ifdef HAVE_LIBPWMD
 	if (temp.proxy && temp.proxy->type == UGET_PROXY_PWMD) {
-		if (uget_plugin_aria2_set_proxy_pwmd(plugin, data, member) == FALSE)
+		if (uget_plugin_aria2_set_proxy_pwmd(plugin, node_info, member) == FALSE)
 			return FALSE;
 	}
 	else
@@ -1110,7 +1115,7 @@ static int  plugin_accept(UgetPluginAria2* plugin, UgData* data)
 		}
 	}
 
-	temp.http = ug_data_get(data, UgetHttpInfo);
+	temp.http = ug_info_get(node_info, UgetHttpInfo);
 	if (temp.http) {
 		if (plugin->uri_part.scheme_len >= 4 &&
 		    strncmp(uri, "http", 4) == 0)
@@ -1142,7 +1147,7 @@ static int  plugin_accept(UgetPluginAria2* plugin, UgData* data)
 		}
 	}
 
-	temp.ftp = ug_data_get(data, UgetFtpInfo);
+	temp.ftp = ug_info_get(node_info, UgetFtpInfo);
 	if (temp.ftp) {
 		if (plugin->uri_part.scheme_len >= 3 && strncmp(uri, "ftp", 3) == 0) {
 			if ((temp.ftp->user     && temp.ftp->user[0]) ||
@@ -1406,7 +1411,7 @@ static void  add_uri_mirrors(UgValue* varray, const char* mirrors)
 // PWMD
 //
 #ifdef HAVE_LIBPWMD
-static gboolean	uget_plugin_aria2_set_proxy_pwmd(UgetPluginAria2 *plugin, UgData* data, UgValue* options)
+static gboolean	uget_plugin_aria2_set_proxy_pwmd(UgetPluginAria2 *plugin, UgInfo* info, UgValue* options)
 {
        struct pwmd_proxy_s pwmd;
        gpg_error_t rc;
@@ -1414,7 +1419,7 @@ static gboolean	uget_plugin_aria2_set_proxy_pwmd(UgetPluginAria2 *plugin, UgData
        UgetProxy *proxy;
 
        memset(&pwmd, 0, sizeof(pwmd));
-       proxy = ug_data_get(data, UgetProxyInfo);
+       proxy = ug_info_get(info, UgetProxyInfo);
        rc = ug_set_pwmd_proxy_options(&pwmd, proxy);
 
        if (rc)
@@ -1453,7 +1458,6 @@ fail:
        gchar *e = g_strdup_printf("Pwmd ERR %i: %s", rc, gpg_strerror(rc));
        message = uget_event_new_error(UGET_EVENT_ERROR_CUSTOM, e);
        uget_plugin_post((UgetPlugin*) plugin, message);
-       fprintf(stderr, "%s\n", e);
        g_free(e);
        return FALSE;
 }
