@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -90,13 +90,13 @@ struct UriLink {
 };
 
 // ----------------------------------------------------------------------------
-// UgetPluginInfo (derived from UgGroupDataInfo)
+// UgetPluginInfo (derived from UgTypeInfo)
 
 static void plugin_init (UgetPluginCurl* plugin);
 static void plugin_final(UgetPluginCurl* plugin);
 static int  plugin_ctrl (UgetPluginCurl* plugin, int code, void* data);
-static int  plugin_accept(UgetPluginCurl* plugin, UgData* data);
-static int  plugin_sync  (UgetPluginCurl* plugin, UgData* data);
+static int  plugin_accept(UgetPluginCurl* plugin, UgInfo* node_info);
+static int  plugin_sync  (UgetPluginCurl* plugin, UgInfo* node_info);
 static UgetResult  global_set(int code, void* parameter);
 static UgetResult  global_get(int code, void* parameter);
 
@@ -223,20 +223,21 @@ static void plugin_init(UgetPluginCurl* plugin)
 static void plugin_final(UgetPluginCurl* plugin)
 {
 	if (plugin->common)
-		ug_group_data_free(plugin->common);
+		ug_data_free(plugin->common);
 	if (plugin->files)
-		ug_group_data_free(plugin->files);
+		ug_data_free(plugin->files);
 	if (plugin->proxy)
-		ug_group_data_free(plugin->proxy);
+		ug_data_free(plugin->proxy);
 	if (plugin->http)
-		ug_group_data_free(plugin->http);
+		ug_data_free(plugin->http);
 	if (plugin->ftp)
-		ug_group_data_free(plugin->ftp);
+		ug_data_free(plugin->ftp);
 	// free uri.list (UriLink), all link will be freed.
 	ug_list_foreach(&plugin->uri.list, (UgForeachFunc) ug_free, NULL);
 
 //	curl_slist_free_all(plugin->ftp_command);
 	ug_free(plugin->folder.path);
+	ug_free(plugin->file.name_fmt);
 	ug_free(plugin->file.path);
 	ug_free(plugin->aria2.path);
 
@@ -317,7 +318,7 @@ static int  plugin_ctrl_speed(UgetPluginCurl* plugin, int* speed)
 // ----------------------------------------------------------------------------
 // plugin_sync
 
-static int  plugin_sync(UgetPluginCurl* plugin, UgData* data)
+static int  plugin_sync(UgetPluginCurl* plugin, UgInfo* node_info)
 {
 	UgetCommon*    common;
 	UgetFiles*     files;
@@ -335,12 +336,13 @@ static int  plugin_sync(UgetPluginCurl* plugin, UgData* data)
 		return FALSE;
 
 	// sync data between plug-in and foreign UgData
-	common = ug_data_realloc(data, UgetCommonInfo);
+	common = ug_info_realloc(node_info, UgetCommonInfo);
 	common->retry_count = plugin->common->retry_count;
-	// sync changed limit from data
+	// sync changed limit from node_info
 	if (plugin->common->max_upload_speed   != common->max_upload_speed ||
 		plugin->common->max_download_speed != common->max_download_speed)
 	{
+		// speed control
 		plugin->common->max_upload_speed   = common->max_upload_speed;
 		plugin->common->max_download_speed = common->max_download_speed;
 		speed[1] = common->max_upload_speed;
@@ -352,7 +354,7 @@ static int  plugin_sync(UgetPluginCurl* plugin, UgData* data)
 	if (common->max_connections > 0)
 		plugin->segment.n_max = common->max_connections;
 
-	progress = ug_data_realloc(data, UgetProgressInfo);
+	progress = ug_info_realloc(node_info, UgetProgressInfo);
 	progress->upload_speed   = plugin->speed.upload;
 	progress->download_speed = plugin->speed.download;
 
@@ -378,7 +380,7 @@ static int  plugin_sync(UgetPluginCurl* plugin, UgData* data)
 	progress->elapsed = time(NULL) - plugin->start_time;
 
 	// update UgetFiles
-	files = ug_data_realloc(data, UgetFilesInfo);
+	files = ug_info_realloc(node_info, UgetFilesInfo);
 	uget_plugin_lock(plugin);
 	uget_files_sync(files, plugin->files);
 	uget_plugin_unlock(plugin);
@@ -414,7 +416,7 @@ static void  plugin_decide_folder(UgetPluginCurl* plugin);
 static void  plugin_decide_files(UgetPluginCurl* plugin);
 static UgThreadResult  plugin_thread(UgetPluginCurl* plugin);
 
-static int  plugin_accept(UgetPluginCurl* plugin, UgData* data)
+static int  plugin_accept(UgetPluginCurl* plugin, UgInfo* node_info)
 {
 	union {
 		UgetCommon*  common;
@@ -423,27 +425,32 @@ static int  plugin_accept(UgetPluginCurl* plugin, UgData* data)
 		UgetHttp*    http;
 		UgetFtp*     ftp;
 	} temp;
+	int  speed[2];
 
-	temp.common = ug_data_get(data, UgetCommonInfo);
+	temp.common = ug_info_get(node_info, UgetCommonInfo);
 	if (temp.common == NULL || temp.common->uri == NULL)
 		return FALSE;
-	plugin->common = ug_group_data_copy(temp.common);
+	plugin->common = ug_data_copy(temp.common);
 	plugin_decide_uris(plugin);
 	plugin_decide_folder(plugin);
+	// speed control: decide speed limit before starting plug-in
+	speed[0] = plugin->limit.download;
+	speed[1] = plugin->limit.upload;
+	plugin_ctrl_speed(plugin, speed);
 
-	temp.files = ug_data_get(data, UgetFilesInfo);
+	temp.files = ug_info_get(node_info, UgetFilesInfo);
 	if (temp.files)
-		plugin->files = ug_group_data_copy(temp.files);
+		plugin->files = ug_data_copy(temp.files);
 	else
-		plugin->files = ug_group_data_new(UgetFilesInfo);
+		plugin->files = ug_data_new(UgetFilesInfo);
 
-	temp.proxy = ug_data_get(data, UgetProxyInfo);
+	temp.proxy = ug_info_get(node_info, UgetProxyInfo);
 	if (temp.proxy)
-		plugin->proxy  = ug_group_data_copy(temp.proxy);
+		plugin->proxy  = ug_data_copy(temp.proxy);
 
-	temp.http = ug_data_get(data, UgetHttpInfo);
+	temp.http = ug_info_get(node_info, UgetHttpInfo);
 	if (temp.http) {
-		plugin->http = ug_group_data_copy(temp.http);
+		plugin->http = ug_data_copy(temp.http);
 		// check http->post_file
 		if (temp.http->post_file) {
 			if (ug_file_is_exist(temp.http->post_file) == FALSE) {
@@ -464,9 +471,9 @@ static int  plugin_accept(UgetPluginCurl* plugin, UgData* data)
 		}
 	}
 
-	temp.ftp = ug_data_get(data, UgetFtpInfo);
+	temp.ftp = ug_info_get(node_info, UgetFtpInfo);
 	if (temp.ftp)
-		plugin->ftp = ug_group_data_copy(temp.ftp);
+		plugin->ftp = ug_data_copy(temp.ftp);
 
 	return TRUE;
 }
@@ -609,6 +616,7 @@ static void  plugin_decide_files(UgetPluginCurl* plugin)
 static void delay_ms(UgetPluginCurl* plugin, int  milliseconds);
 static int  switch_uri(UgetPluginCurl* plugin, UgetCurl* ugcurl, int is_resumable);
 static int  prepare_file(UgetCurl* ugcurl, UgetPluginCurl* plugin);
+static char* get_repeating_fmt_string(char* filename);
 static void complete_file(UgetPluginCurl* plugin);
 static int  load_file_info(UgetPluginCurl* plugin);
 static void clear_file_info(UgetPluginCurl* plugin);
@@ -1031,6 +1039,7 @@ static int prepare_file(UgetCurl* ugcurl, UgetPluginCurl* plugin)
 	if (plugin->folder.path)
 		strcpy(plugin->file.path, plugin->folder.path);
 	strcat(plugin->file.path, common->file);
+	plugin->file.name_fmt = get_repeating_fmt_string(common->file);
 
 	// create folder
 	if (ug_create_dir_all(plugin->file.path, plugin->folder.length) == -1) {
@@ -1119,8 +1128,8 @@ static int prepare_file(UgetCurl* ugcurl, UgetPluginCurl* plugin)
 			break;
 		}
 
-		// filename repeat
-		sprintf(plugin->file.path + length, ".%d", counts);
+		sprintf(plugin->file.path + plugin->folder.length,
+				plugin->file.name_fmt, counts);
 	}
 
 	if (counts == MAX_REPEAT_COUNTS) {
@@ -1200,6 +1209,94 @@ static int prepare_file(UgetCurl* ugcurl, UgetPluginCurl* plugin)
 	}
 }
 
+// used by get_repeating_fmt_string()
+enum {
+	EXT_NUMBER = 0x01,
+	EXT_UPPER  = 0x02,
+	EXT_LOWER  = 0x04,
+
+	EXT_CAMEL  = EXT_UPPER | EXT_LOWER,
+	EXT_ALL    = EXT_UPPER | EXT_LOWER | EXT_NUMBER,
+};
+
+// used by get_repeating_fmt_string()
+static char* find_dot_ext(const char* filename, const char* end, uint8_t* status)
+{
+	uint8_t     stat = 0;
+    const char* cur = NULL;
+
+	for (cur = end-1;  cur >= filename;  cur--) {
+		if (cur[0] == '.') {
+			if (cur == end-1 || cur == filename)
+				break;
+			if (status)
+				status[0] = stat;
+			return (char*)cur;
+		}
+		else if (cur[0] >= '0' && cur[0] <= '9')
+			stat |= EXT_NUMBER;
+		else if (cur[0] >= 'A' && cur[0] <= 'Z')
+			stat |= EXT_UPPER;
+		else if (cur[0] >= 'a' && cur[0] <= 'z')
+			stat |= EXT_LOWER;
+		else
+			break;
+	}
+	return NULL;
+}
+
+static char* get_repeating_fmt_string(char* filename)
+{
+	char*   result;
+	char*   dot[3] = {NULL, NULL, NULL};
+	uint8_t stat[3] = {0, 0, 0};
+	int     index;
+	int     length;
+
+	length = strlen(filename);
+	result = ug_malloc(length + 3 + 1);  // + ".%d" + "\0"
+
+	// dot[0]
+	dot[0] = find_dot_ext(filename, filename + length, &stat[0]);
+	if (dot[0])
+		dot[1] = find_dot_ext(filename, dot[0], &stat[1]);
+	if (dot[1])
+		dot[2] = find_dot_ext(filename, dot[1], &stat[2]);
+
+	for (index = 2;  index >= 0;  index--) {
+		if (dot[index] == NULL)
+			continue;
+
+		// replace exist number
+		if (stat[index] == EXT_NUMBER) {
+			// strncpy() doesn't always null-terminated
+			strncpy(result, filename, dot[index] - filename);
+			result[dot[index] - filename] = 0;
+			strcat(result, ".%d");
+			if (index > 0)
+				strcat(result, dot[index-1]);
+			return result;
+		}
+
+		// insert number
+		if (index == 2)
+			continue;
+		// strncpy() doesn't always null-terminated
+		strncpy(result, filename, dot[index] - filename);
+		result[dot[index] - filename] = 0;
+		strcat(result, ".%d");
+		strcat(result, dot[index]);
+		return result;
+	}
+
+	// append number
+	strcpy(result, filename);
+	strcat(result, ".%d");
+
+	// return printf() format string
+	return result;
+}
+
 static int  load_file_info(UgetPluginCurl* plugin)
 {
 	UgetCommon*  common;
