@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2016-2018 by C.H. Huang
+ *   Copyright (C) 2016-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -65,13 +65,13 @@
 #endif
 
 // ----------------------------------------------------------------------------
-// UgetPluginInfo (derived from UgGroupDataInfo)
+// UgetPluginInfo (derived from UgTypeInfo)
 
 static void plugin_init (UgetPluginMedia* plugin);
 static void plugin_final(UgetPluginMedia* plugin);
 static int  plugin_ctrl (UgetPluginMedia* plugin, int code, void* data);
-static int  plugin_accept(UgetPluginMedia* plugin, UgData* data);
-static int  plugin_sync  (UgetPluginMedia* plugin, UgData* data);
+static int  plugin_accept(UgetPluginMedia* plugin, UgInfo* node_info);
+static int  plugin_sync  (UgetPluginMedia* plugin, UgInfo* node_info);
 static UgetResult  global_set(int code, void* parameter);
 static UgetResult  global_get(int code, void* parameter);
 
@@ -170,7 +170,7 @@ static int  plugin_ctrl(UgetPluginMedia* plugin, int code, void* data)
 {
 	switch (code) {
 	case UGET_PLUGIN_CTRL_START:
-		if (plugin->target_data) {
+		if (plugin->target_info) {
 			return uget_plugin_agent_start((UgetPluginAgent*)plugin,
 			                               (UgThreadFunc)plugin_thread);
 		}
@@ -186,7 +186,7 @@ static int  plugin_ctrl(UgetPluginMedia* plugin, int code, void* data)
 // ----------------------------------------------------------------------------
 // plugin_sync
 
-static int  plugin_sync(UgetPluginMedia* plugin, UgData* data)
+static int  plugin_sync(UgetPluginMedia* plugin, UgInfo* node_info)
 {
 	UgetFiles*     files;
 	UgetCommon*    common;
@@ -198,10 +198,10 @@ static int  plugin_sync(UgetPluginMedia* plugin, UgData* data)
 		plugin->synced = TRUE;
 	}
 	// avoid crash if plug-in failed to start.
-	if (plugin->target_data == NULL)
+	if (plugin->target_info == NULL)
 		return FALSE;
 	// sync data between plug-in and foreign UgData
-	common = ug_data_realloc(data, UgetCommonInfo);
+	common = ug_info_realloc(node_info, UgetCommonInfo);
 	// sum retry count
 	common->retry_count = plugin->target_common->retry_count + plugin->retry_count;
 
@@ -220,7 +220,7 @@ static int  plugin_sync(UgetPluginMedia* plugin, UgData* data)
 			common->name = ug_strdup(plugin->title);
 	}
 
-	// sync common data (include speed limit) between foreign data and target_data
+	// sync common data (include speed limit) between foreign data and target_info
 	uget_plugin_agent_sync_common((UgetPluginAgent*) plugin,
 	                              common, plugin->target_common);
 	// downloading file name changed
@@ -232,13 +232,13 @@ static int  plugin_sync(UgetPluginMedia* plugin, UgData* data)
 		uget_plugin_unlock(plugin);
 	}
 	// update UgetFiles
-	files = ug_data_realloc(data, UgetFilesInfo);
+	files = ug_info_realloc(node_info, UgetFilesInfo);
 	uget_plugin_lock(plugin);
     uget_files_sync(files, plugin->target_files);
     uget_plugin_unlock(plugin);
 
-	// sync progress data from target_data to foreign data
-	progress = ug_data_realloc(data, UgetProgressInfo);
+	// sync progress data from target_info to foreign data
+	progress = ug_info_realloc(node_info, UgetProgressInfo);
 	uget_plugin_agent_sync_progress((UgetPluginAgent*) plugin,
 	                                progress, plugin->target_progress);
 	// recount progress if plug-in download multiple files
@@ -264,20 +264,20 @@ static int  plugin_sync(UgetPluginMedia* plugin, UgData* data)
 
 // ----------------------------------------------------------------------------
 
-static int  plugin_accept(UgetPluginMedia* plugin, UgData* data)
+static int  plugin_accept(UgetPluginMedia* plugin, UgInfo* node_info)
 {
 	UgetCommon*  common;
 
-	common = ug_data_get(data, UgetCommonInfo);
+	common = ug_info_get(node_info, UgetCommonInfo);
 	if (common == NULL || common->uri == NULL)
 		return FALSE;
 
-	plugin->target_data = ug_data_new(8, 0);
-	ug_data_assign(plugin->target_data, data, NULL);
-	plugin->target_files  = ug_data_realloc(plugin->target_data, UgetFilesInfo);
-	plugin->target_common = ug_data_get(plugin->target_data, UgetCommonInfo);
-	plugin->target_proxy  = ug_data_get(plugin->target_data, UgetProxyInfo);
-	plugin->target_progress = ug_data_realloc(plugin->target_data, UgetProgressInfo);
+	plugin->target_info = ug_info_new(8, 0);
+	ug_info_assign(plugin->target_info, node_info, NULL);
+	plugin->target_files  = ug_info_realloc(plugin->target_info, UgetFilesInfo);
+	plugin->target_common = ug_info_get(plugin->target_info, UgetCommonInfo);
+	plugin->target_proxy  = ug_info_get(plugin->target_info, UgetProxyInfo);
+	plugin->target_progress = ug_info_realloc(plugin->target_info, UgetProgressInfo);
 
 	return TRUE;
 }
@@ -331,7 +331,7 @@ static UgThreadResult  plugin_thread(UgetPluginMedia* plugin)
 			ug_list_position((UgList*) umedia, (UgLink*) umitem);
 
 	// set HTTP referrer
-	http = ug_data_realloc(plugin->target_data, UgetHttpInfo);
+	http = ug_info_realloc(plugin->target_info, UgetHttpInfo);
 	if (http->referrer == NULL)
 		http->referrer = ug_strdup_printf("%s%s", common->uri, "# ");
 	// clear copied common URI
@@ -418,7 +418,7 @@ static UgThreadResult  plugin_thread(UgetPluginMedia* plugin)
 		                             &plugin_info);
 		// create target_plugin to download
 		plugin->target_plugin = uget_plugin_new(plugin_info);
-		uget_plugin_accept(plugin->target_plugin, plugin->target_data);
+		uget_plugin_accept(plugin->target_plugin, plugin->target_info);
 		uget_plugin_ctrl_speed(plugin->target_plugin, plugin->limit);
 		if (uget_plugin_start(plugin->target_plugin) == FALSE) {
 			msg = uget_event_new_error(UGET_EVENT_ERROR_THREAD_CREATE_FAILED,
@@ -471,7 +471,7 @@ static UgThreadResult  plugin_thread(UgetPluginMedia* plugin)
 			plugin->synced = FALSE;
 			uget_plugin_lock(plugin);
 			uget_plugin_sync(plugin->target_plugin,
-			                 plugin->target_data);
+			                 plugin->target_info);
 			uget_plugin_unlock(plugin);
 		} while (uget_plugin_get_state(plugin->target_plugin));
 
