@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2016-2018 by C.H. Huang
+ *   Copyright (C) 2016-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -44,9 +44,9 @@
 extern "C" {
 #endif
 
-typedef struct	UgetPluginMedia   UgetPluginMedia;
+typedef struct UgetPluginMedia          UgetPluginMedia;
 
-extern const    UgetPluginInfo*   UgetPluginMediaInfo;
+extern  const  UgetPluginInfo*          UgetPluginMediaInfo;
 
 typedef enum {
 	UGET_PLUGIN_MEDIA_GLOBAL = UGET_PLUGIN_AGENT_GLOBAL_DERIVED,    // begin
@@ -83,11 +83,11 @@ struct UgetPluginMedia
 
 	// ------ UgetPluginAgent members ------
 	// This plug-in use other plug-in to download files,
-	// so we need extra UgetPlugin and UgData.
+	// so we need extra UgetPlugin and UgInfo.
 
-	// plugin->target_data is a copy of UgData that store in UgetApp
-	UgData*       target_data;
-	// target_plugin use target_data to download
+	// plugin->target_info is a copy of UgInfo that store in UgetApp
+	UgInfo*       target_info;
+	// target_plugin use target_info to download
 	UgetPlugin*   target_plugin;
 
 	// speed limit control
@@ -105,7 +105,7 @@ struct UgetPluginMedia
 	uint8_t       named:1;          // change UgetCommon::name by title
 	uint8_t       file_renamed:1;   // downloading filename changed
 
-	// These UgGroupData store in plugin->target_data
+	// These UgData store in plugin->target_info
 	UgetFiles*    target_files;
 	UgetProxy*    target_proxy;
 	UgetCommon*   target_common;
@@ -134,12 +134,18 @@ struct UgetPluginMedia
 namespace Uget
 {
 
+const PluginInfo* const PluginMediaInfo = (const PluginInfo*) UgetPluginMediaInfo;
+
 // This one is for derived use only. No data members here.
 // Your derived struct/class must be C++11 standard-layout
-struct PluginMediaMethod : Uget::PluginMethod {};
+struct PluginMediaMethod : PluginAgentMethod {};
 
 // This one is for directly use only. You can NOT derived it.
-struct PluginMedia : Uget::PluginMediaMethod, UgetPluginMedia {};
+struct PluginMedia : PluginMediaMethod, UgetPluginMedia
+{
+	inline void* operator new(size_t size)
+		{ return uget_plugin_new(PluginMediaInfo); }
+};
 
 };  // namespace Uget
 
