@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2016-2018 by C.H. Huang
+ *   Copyright (C) 2016-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -47,9 +47,9 @@
 extern "C" {
 #endif
 
-typedef struct	UgetPluginMega   UgetPluginMega;
+typedef struct UgetPluginMega     UgetPluginMega;
 
-extern const    UgetPluginInfo*  UgetPluginMegaInfo;
+extern  const  UgetPluginInfo*    UgetPluginMegaInfo;
 
 /* ----------------------------------------------------------------------------
    UgetPluginMega: It derived from UgetPluginAgent.
@@ -78,11 +78,11 @@ struct UgetPluginMega
 
 	// ------ UgetPluginAgent members ------
 	// This plug-in use other plug-in to download files,
-	// so we need extra UgetPlugin and UgData.
+	// so we need extra UgetPlugin and UgInfo.
 
-	// plugin->target_data is a copy of UgData that store in UgetApp
-	UgData*       target_data;
-	// target_plugin use target_data to download
+	// plugin->target_info is a copy of UgInfo that store in UgetApp
+	UgInfo*       target_info;
+	// target_plugin use target_info to download
 	UgetPlugin*   target_plugin;
 
 	// speed limit control
@@ -100,7 +100,7 @@ struct UgetPluginMega
 	uint8_t       synced:1;         // used by plugin_sync()
 	uint8_t       decrypting:1;     // decrypting downloaded file
 
-	// These UgGroupData store in target_data
+	// These UgData store in target_info
 	UgetFiles*    target_files;
 	UgetProxy*    target_proxy;
 	UgetCommon*   target_common;
@@ -120,9 +120,35 @@ struct UgetPluginMega
 	UgValue  value;
 };
 
-
 #ifdef __cplusplus
 }
 #endif
 
+// ----------------------------------------------------------------------------
+// C++11 standard-layout
+
+#ifdef __cplusplus
+
+namespace Uget
+{
+
+const PluginInfo* const PluginMegaInfo = (const PluginInfo*) UgetPluginMegaInfo;
+
+// This one is for derived use only. No data members here.
+// Your derived struct/class must be C++11 standard-layout
+struct PluginMegaMethod : PluginAgentMethod {};
+
+// This one is for directly use only. You can NOT derived it.
+struct PluginMega : PluginMegaMethod, UgetPluginMega
+{
+	inline void* operator new(size_t size)
+		{ return uget_plugin_new(PluginMegaInfo); }
+};
+
+
+};  // namespace Uget
+
+#endif  // __cplusplus
+
+
 #endif  // End of UGET_PLUGIN_MEGA_H
