@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -51,6 +51,11 @@ void  uget_task_init(UgetTask* task)
 		task->watch[count].func = NULL;
 		task->watch[count].data = NULL;
 	}
+	// current speed & speed limit
+	task->speed.download = 0;
+	task->speed.upload   = 0;
+	task->limit.download = 0;
+	task->limit.upload   = 0;
 }
 
 void  uget_task_final(UgetTask* task)
@@ -71,12 +76,12 @@ int   uget_task_add(UgetTask* task, UgetNode* node, const UgetPluginInfo* info)
 	} temp;
 
 	// UgetRelation: check exist plug-in
-	relation = ug_data_realloc(node->data, UgetRelationInfo);
-	if (relation->task.plugin)
+	relation = ug_info_realloc(node->info, UgetRelationInfo);
+	if (relation->task)
 		return FALSE;
 
 	// UgetProgress: clear progress when it completed
-	temp.progress = ug_data_get(node->data, UgetProgressInfo);
+	temp.progress = ug_info_get(node->info, UgetProgressInfo);
 	if (temp.progress) {
 		// reset progress if it's percent is 100%.
 		if (temp.progress->percent == 100) {
@@ -90,13 +95,14 @@ int   uget_task_add(UgetTask* task, UgetNode* node, const UgetPluginInfo* info)
 	}
 
 	// UgetCommon: clear retry_count
-	temp.common = ug_data_get(node->data, UgetCommonInfo);
+	temp.common = ug_info_get(node->info, UgetCommonInfo);
 	if (temp.common)
 		temp.common->retry_count = 0;
 
 	// create plug-in and control it
-	relation->task.plugin = uget_plugin_new(info);
-	uget_plugin_accept(relation->task.plugin, node->data);
+	relation->task = ug_malloc0(sizeof(struct UgetRelationTask));
+	relation->task->plugin = uget_plugin_new(info);
+	uget_plugin_accept(relation->task->plugin, node->info);
 	if (task->limit.download || task->limit.upload) {
 		// backup current speed limit
 		temp_int_array[0] = task->limit.download;
@@ -108,20 +114,21 @@ int   uget_task_add(UgetTask* task, UgetNode* node, const UgetPluginInfo* info)
 		                    temp_int_array[0] - dlul_int_array[0],
 		                    temp_int_array[1] - dlul_int_array[1]);
 		// set speed limit for new task
-		uget_plugin_ctrl_speed(relation->task.plugin, dlul_int_array);
+		uget_plugin_ctrl_speed(relation->task->plugin, dlul_int_array);
 		// restore current speed limit
 		task->limit.download = temp_int_array[0];
 		task->limit.upload   = temp_int_array[1];
 	}
-	if (uget_plugin_start(relation->task.plugin) == FALSE) {
+	if (uget_plugin_start(relation->task->plugin) == FALSE) {
 		// dispatch error message from plug-in
-		uget_task_dispatch1(task, node, relation->task.plugin);
+		uget_task_dispatch1(task, node, relation->task->plugin);
 		// release plug-in
-		uget_plugin_unref(relation->task.plugin);
-		relation->task.plugin = NULL;
+		uget_plugin_unref(relation->task->plugin);
+		// free task runtime data
+		ug_free(relation->task);
+		relation->task = NULL;
 		return FALSE;
 	}
-	relation->task.plugin_name = ug_strdup(info->name);
 
 	ug_slinks_add((UgSLinks*) task, node);
 	return TRUE;
@@ -135,14 +142,16 @@ int  uget_task_remove(UgetTask* task, UgetNode* node)
 	if (ug_slinks_find((UgSLinks*) task, node, &prev)) {
 		ug_slinks_remove((UgSLinks*) task, node, prev);
 		// UgetRelation
-		relation = ug_data_get(node->data, UgetRelationInfo);
+		relation = ug_info_get(node->info, UgetRelationInfo);
 		if (relation) {
-//			uget_plugin_post(relation->task.plugin,
+//			uget_plugin_post(relation->task->plugin,
 //					uget_event_new_state(node, UGET_GROUP_QUEUING));
-			uget_plugin_stop(relation->task.plugin);
-			uget_plugin_unref(relation->task.plugin);
-			relation->task.plugin = NULL;
+			uget_plugin_stop(relation->task->plugin);
+			uget_plugin_unref(relation->task->plugin);
 			relation->group &= ~UGET_GROUP_ACTIVE;
+			// free task runtime data
+			ug_free(relation->task);
+			relation->task = NULL;
 		}
 		return TRUE;
 	}
@@ -167,13 +176,13 @@ static int  uget_task_dispatch1(UgetTask* task, UgetNode* node, UgetPlugin* plug
 		UgetFiles*    files;
 	} temp;
 
-	active = uget_plugin_sync(plugin, node->data);
+	active = uget_plugin_sync(plugin, node->info);
 	// update UgetFiles
-	temp.files = ug_data_get(node->data, UgetFilesInfo);
+	temp.files = ug_info_get(node->info, UgetFilesInfo);
 	if (temp.files)
 		uget_files_erase_deleted(temp.files);
 	// plug-in was paused by user (see function uget_app_pause_download)
-	relation = ug_data_realloc(node->data, UgetRelationInfo);
+	relation = ug_info_realloc(node->info, UgetRelationInfo);
 	if (relation->group & UGET_GROUP_PAUSED)
 		active = FALSE;
 	// plug-in has stopped if uget_plugin_sync() return FALSE.
@@ -199,7 +208,7 @@ static int  uget_task_dispatch1(UgetTask* task, UgetNode* node, UgetPlugin* plug
 			relation->group |= UGET_GROUP_ERROR;  // don't break here
 		case UGET_EVENT_WARNING:
 		case UGET_EVENT_NORMAL:
-			temp.log = ug_data_realloc(node->data, UgetLogInfo);
+			temp.log = ug_info_realloc(node->info, UgetLogInfo);
 			ug_list_prepend(&temp.log->messages, (UgLink*) event);
 			break;
 
@@ -250,16 +259,16 @@ void  uget_task_dispatch(UgetTask* task)
 
 	for (link = task->used;  link;  link = link->next) {
 		node = link->data;
-		relation = ug_data_get(node->data, UgetRelationInfo);
-		if (uget_task_dispatch1(task, node, relation->task.plugin) == FALSE)
+		relation = ug_info_get(node->info, UgetRelationInfo);
+		if (uget_task_dispatch1(task, node, relation->task->plugin) == FALSE)
 			continue;
 		// speed
-		progress = ug_data_get(node->data, UgetProgressInfo);
+		progress = ug_info_get(node->info, UgetProgressInfo);
 		if (progress) {
 			task->speed.download += progress->download_speed;
 			task->speed.upload   += progress->upload_speed;
-			relation->task.speed[0] = progress->download_speed;
-			relation->task.speed[1] = progress->upload_speed;
+			relation->task->speed[0] = progress->download_speed;
+			relation->task->speed[1] = progress->upload_speed;
 		}
 	}
 }
@@ -325,22 +334,22 @@ static void uget_task_adjust_speed_index(UgetTask* task, int idx, int remain)
 		// increase speed by priority
 		for (link = task->used;  link;  link = link->next) {
 			node = (UgetNode*) link->data;
-			relation = ug_data_get(node->data, UgetRelationInfo);
-			relation->task.prev = prev;
+			relation = ug_info_get(node->info, UgetRelationInfo);
+			relation->task->prev = prev;
 			prev = relation;
-			n_piece += relation->task.priority + 1;
+			n_piece += relation->priority + 1;
 		}
 
 		remain = remain / n_piece;
 		for (;  relation;  relation = prev) {
-			relation->task.limit[idx] = relation->task.speed[idx] +
-			                            remain * (relation->task.priority+1);
-			if (relation->task.limit[idx] < SPEED_MIN)
-				relation->task.limit[idx] = SPEED_MIN;
-			uget_plugin_ctrl_speed(relation->task.plugin,
-			                       relation->task.limit);
-			prev = relation->task.prev;
-			relation->task.prev = NULL;
+			relation->task->limit[idx] = relation->task->speed[idx] +
+			                            remain * (relation->priority+1);
+			if (relation->task->limit[idx] < SPEED_MIN)
+				relation->task->limit[idx] = SPEED_MIN;
+			uget_plugin_ctrl_speed(relation->task->plugin,
+			                       relation->task->limit);
+			prev = relation->task->prev;
+			relation->task->prev = NULL;
 		}
 	}
 	else {
@@ -348,12 +357,12 @@ static void uget_task_adjust_speed_index(UgetTask* task, int idx, int remain)
 		remain = remain / task->n_links;
 		for (link = task->used;  link;  link = link->next) {
 			node = (UgetNode*) link->data;
-			relation = ug_data_get(node->data, UgetRelationInfo);
-			relation->task.limit[idx] = relation->task.speed[idx] + remain;
-			if (relation->task.limit[idx] < SPEED_MIN)
-				relation->task.limit[idx] = SPEED_MIN;
-			uget_plugin_ctrl_speed(relation->task.plugin,
-			                       relation->task.limit);
+			relation = ug_info_get(node->info, UgetRelationInfo);
+			relation->task->limit[idx] = relation->task->speed[idx] + remain;
+			if (relation->task->limit[idx] < SPEED_MIN)
+				relation->task->limit[idx] = SPEED_MIN;
+			uget_plugin_ctrl_speed(relation->task->plugin,
+			                       relation->task->limit);
 		}
 	}
 }
@@ -366,9 +375,9 @@ static void uget_task_disable_limit_index(UgetTask* task, int idx)
 
 	for (link = task->used;  link;  link = link->next) {
 		node = (UgetNode*) link->data;
-		relation = ug_data_get(node->data, UgetRelationInfo);
-		relation->task.limit[idx] = 0;
-		uget_plugin_ctrl_speed(relation->task.plugin,
-		                       relation->task.limit);
+		relation = ug_info_get(node->info, UgetRelationInfo);
+		relation->task->limit[idx] = 0;
+		uget_plugin_ctrl_speed(relation->task->plugin,
+		                       relation->task->limit);
 	}
 }
