@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -51,13 +51,17 @@
 extern "C" {
 #endif
 
+/* UgetTask - match/manage UgetNode and UgetPlugin
+            - adjust/limit speed
+ */
+
 typedef struct  UgetTask        UgetTask;
 
 typedef int (*UgetWatchFunc) (void* instance, UgetEvent* msg,
                               UgetNode* node, void* data);
 
 // ----------------------------------------------------------------------------
-// UgetTask : match UgetNode and UgetPlugin
+// UgetTask functions
 
 void  uget_task_init(UgetTask* task);
 void  uget_task_final(UgetTask* task);
@@ -75,6 +79,9 @@ void  uget_task_adjust_speed(UgetTask* task);
 }
 #endif
 
+// ----------------------------------------------------------------------------
+// UgetTask structure
+
 struct UgetTask
 {
 	UG_SLINKS_MEMBERS;
@@ -94,8 +101,8 @@ struct UgetTask
 	} watch[UGET_TASK_N_WATCH];
 
 	struct {
-		int  upload;
-		int  download;
+		int   upload;
+		int   download;
 	} speed, limit;
 
 #ifdef __cplusplus
@@ -112,10 +119,13 @@ struct UgetTask
 	inline void dispatch(void)
 		{ uget_task_dispatch((UgetTask*) this); }
 
+	inline void setSpeed(int dl_speed, int ul_speed)
+		{ uget_task_set_speed(this, dl_speed, ul_speed); }
+	inline void adjustSpeed(void)
+		{ uget_task_adjust_speed(this); }
 #endif  // __cplusplus
 };
 
-
 // ----------------------------------------------------------------------------
 // C++11 standard-layout
 
