@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -313,6 +313,7 @@ int   ug_file_is_dir (const char* dir)
 
 #endif // _WIN32 || _WIN64
 
+#ifndef USE__ANDROID__SAF
 int  ug_create_dir_all (const char* dir, int len)
 {
 	const char*   dir_end;
@@ -353,8 +354,8 @@ int  ug_create_dir_all (const char* dir, int len)
 
 		if (element_os == NULL)
 			break;
-		if (ug_create_dir (element_os) == -1) {
-			if (ug_file_is_exist (element_os) == FALSE) {
+		if (ug_file_is_exist (element_os) == FALSE) {
+			if (ug_create_dir (element_os) == -1) {
 				ug_free (element_os);
 				return -1;
 			}
@@ -363,6 +364,7 @@ int  ug_create_dir_all (const char* dir, int len)
 	}
 	return -1;
 }
+#endif  // USE__ANDROID__SAF
 
 // ----------------------------------------------------------------------------
 // File I/O
