@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -40,7 +40,7 @@
 #include <stdint.h>
 #include <UgArray.h>
 #include <UgEntry.h>
-#include <UgData.h>
+#include <UgInfo.h>
 
 #ifdef __cplusplus
 extern "C" {
