@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2018 by C.H. Huang
+ *   Copyright (C) 2005-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -298,6 +298,17 @@ char* ug_uri_get_file (UgUri* uuri)
 	return ug_strndup (str, len);
 }
 
+int  ug_uri_is_file(UgUri* uuri)
+{
+	if (uuri->scheme_len == 4) {
+		if (strncmp(uuri->uri, "file", 4) != 0)
+			return FALSE;
+	}
+	else if (uuri->scheme_len != 0)
+		return FALSE;
+	return TRUE;
+}
+
 // ------------------------------------
 // match uri functions
 
