@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2013-2018 by C.H. Huang
+ *   Copyright (C) 2013-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -203,7 +203,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.relation = ug_dataset_get (dataset, UgRelationInfo, 0);
 	if (old.relation) {
-		new.relation = ug_data_realloc (node->data, UgetRelationInfo);
+		new.relation = ug_info_realloc (node->info, UgetRelationInfo);
 		if (old.relation->hints & UG_HINT_PAUSED)
 			new.relation->group |= UGET_GROUP_PAUSED;
 		if (old.relation->hints & UG_HINT_ERROR)
@@ -221,7 +221,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.common = ug_dataset_get (dataset, UgCommonInfo, 0);
 	if (old.common) {
-		new.common = ug_data_realloc (node->data, UgetCommonInfo);
+		new.common = ug_info_realloc (node->info, UgetCommonInfo);
 		new.common->name = old.common->name;
 		old.common->name = NULL;
 		if (new.common->name == NULL && old.common->file)
@@ -251,7 +251,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.proxy = ug_dataset_get (dataset, UgProxyInfo, 0);
 	if (old.proxy) {
-		new.proxy = ug_data_realloc (node->data, UgetProxyInfo);
+		new.proxy = ug_info_realloc (node->info, UgetProxyInfo);
 		new.proxy->host = old.proxy->host;
 		old.proxy->host = NULL;
 		new.proxy->port = old.proxy->port;
@@ -264,7 +264,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.http = ug_dataset_get (dataset, UgHttpInfo, 0);
 	if (old.http) {
-		new.http = ug_data_realloc (node->data, UgetHttpInfo);
+		new.http = ug_info_realloc (node->info, UgetHttpInfo);
 		new.http->user = old.http->user;
 		old.http->user = NULL;
 		new.http->password = old.http->password;
@@ -287,7 +287,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.ftp = ug_dataset_get (dataset, UgFtpInfo, 0);
 	if (old.ftp) {
-		new.ftp = ug_data_realloc (node->data, UgetFtpInfo);
+		new.ftp = ug_info_realloc (node->info, UgetFtpInfo);
 		new.ftp->user = old.ftp->user;
 		old.ftp->user = NULL;
 		new.ftp->password = old.ftp->password;
@@ -297,7 +297,7 @@ static void uget_node_set_by_dataset (UgetNode* node, UgDataset* dataset)
 
 	old.progress = ug_dataset_get (dataset, UgProgressInfo, 0);
 	if (old.progress) {
-		new.progress = ug_data_realloc (node->data, UgetProgressInfo);
+		new.progress = ug_info_realloc (node->info, UgetProgressInfo);
 		new.progress->complete = old.progress->complete;
 		new.progress->total = old.progress->total;
 		new.progress->complete = old.progress->complete;
@@ -319,11 +319,11 @@ static UgetNode* uget_node_from_category (UgCategory* category1)
 
 	node = uget_node_new (NULL);
 	uget_node_set_by_dataset (node, category1->defaults);
-	category = ug_data_realloc (node->data, UgetCategoryInfo);
+	category = ug_info_realloc (node->info, UgetCategoryInfo);
 	category->active_limit   = category1->active_limit;
 	category->finished_limit = category1->finished_limit;
 	category->recycled_limit = category1->recycled_limit;
-	common = ug_data_realloc(node->data, UgetCommonInfo);
+	common = ug_info_realloc(node->info, UgetCommonInfo);
 	common->name = category1->name;
 	category1->name = NULL;
 	// other
