@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -106,9 +106,9 @@ static void  on_download_cursor_changed (GtkTreeView* view, UgtkApp* app)
 	if (node == NULL)
 		priority = UGET_PRIORITY_NORMAL;
 	else {
-		relation = ug_data_get (node->data, UgetRelationInfo);
+		relation = ug_info_get (node->info, UgetRelationInfo);
 		if (relation)
-			priority = relation->task.priority;
+			priority = relation->priority;
 		else
 			priority = UGET_PRIORITY_NORMAL;
 	}
@@ -482,7 +482,7 @@ static void node_inserted (UgetNode* node, UgetNode* sibling, UgetNode* child)
 	GtkTreeIter  iter;
 	UgtkApp*     app;
 
-	app = node->control->data;
+	app = node->control->notifier->data;
 	if (node == (UgetNode*) app->traveler.category.model->root) {
 		// category inserted
 		path = gtk_tree_path_new_from_indices (
@@ -513,7 +513,7 @@ static void node_removed (UgetNode* node, UgetNode* sibling, UgetNode* child)
 	UgtkApp*     app;
 	int          pos;
 
-	app = node->control->data;
+	app = node->control->notifier->data;
 	if (node == (UgetNode*) app->traveler.category.model->root) {
 		// category removed
 		if (sibling)
@@ -549,7 +549,7 @@ static void node_updated (UgetNode* child)
 	UgtkApp*     app;
 
 	node = child->parent;
-	app = node->control->data;
+	app = node->control->notifier->data;
 	if (node == (UgetNode*) app->traveler.category.model->root) {
 		// category changed
 		path = gtk_tree_path_new_from_indices (
