@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2018 by C.H. Huang
+ *   Copyright (C) 2005-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -198,7 +198,7 @@ static void on_no_batch_response (UgtkBatchDialog* bdialog)
 	uri = gtk_entry_get_text ((GtkEntry*)bdialog->download.uri_entry);
 	if (ugtk_node_dialog_confirm_existing((UgtkNodeDialog*) bdialog, uri)) {
 		dnode = uget_node_new (NULL);
-		ugtk_node_dialog_get ((UgtkNodeDialog*) bdialog, dnode->data);
+		ugtk_node_dialog_get ((UgtkNodeDialog*) bdialog, dnode->info);
 		uget_app_add_download ((UgetApp*) app, dnode, cnode, FALSE);
 	}
 }
@@ -222,14 +222,13 @@ static void on_sequencer_response (UgtkBatchDialog* bdialog)
 
 	for (link = result.head;  link;  link = link->next) {
 		dnode = uget_node_new (NULL);
-		common = ug_data_realloc (dnode->data, UgetCommonInfo);
-		ugtk_node_dialog_get ((UgtkNodeDialog*) bdialog, dnode->data);
+		common = ug_info_realloc (dnode->info, UgetCommonInfo);
+		ugtk_node_dialog_get ((UgtkNodeDialog*) bdialog, dnode->info);
 		common->uri = ug_strdup (link->data);
 		uget_app_add_download ((UgetApp*) app, dnode, cnode, FALSE);
 	}
 
-	ug_list_foreach_link (&result, (UgForeachFunc)ug_free, NULL);
-	ug_list_clear (&result, FALSE);
+	uget_sequence_clear_result(&result);
 }
 
 static void on_selector_response (UgtkBatchDialog* bdialog)
@@ -250,8 +249,8 @@ static void on_selector_response (UgtkBatchDialog* bdialog)
 
 	for (link = uri_list;  link;  link = link->next) {
 		dnode = uget_node_new (NULL);
-		common = ug_data_realloc (dnode->data, UgetCommonInfo);
-		ugtk_node_dialog_get ((UgtkNodeDialog*) bdialog, dnode->data);
+		common = ug_info_realloc (dnode->info, UgetCommonInfo);
+		ugtk_node_dialog_get ((UgtkNodeDialog*) bdialog, dnode->info);
 #if 0
 		common->uri = link->data;
 		link->data = NULL;
