@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2018 by C.H. Huang
+ *   Copyright (C) 2012-2019 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -99,7 +99,7 @@ static void col_set_icon (GtkTreeViewColumn *tree_column,
 	// select icon_name
 	for (index = 0;  index < state_icon_pair_len;  index++) {
 		key = (intptr_t)state_icon_pair[index].key;
-		relation = ug_data_realloc(node->data, UgetRelationInfo);
+		relation = ug_info_realloc(node->info, UgetRelationInfo);
 		if ((key & relation->group) == key) {
 			icon_name = state_icon_pair[index].data;
 			break;
@@ -143,12 +143,12 @@ static void col_set_name (GtkTreeViewColumn *tree_column,
 
 	node = node->base;
 	name = _("unnamed");
-	common = ug_data_get(node->data, UgetCommonInfo);
+	common = ug_info_get(node->info, UgetCommonInfo);
 
 	if (common && common->name)
 		name = common->name;
 	else {
-		common = ug_data_get (node->data, UgetCommonInfo);
+		common = ug_info_get (node->info, UgetCommonInfo);
 		if (common) {
 			if (common->file)
 				name = common->file;
@@ -176,7 +176,7 @@ static void col_set_complete (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	progress = ug_data_get (node->data, UgetProgressInfo);
+	progress = ug_info_get (node->info, UgetProgressInfo);
 	if (progress && progress->total)
 		string = ug_str_from_int_unit (progress->complete, NULL);
 	else
@@ -203,7 +203,7 @@ static void col_set_total (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	progress = ug_data_get (node->data, UgetProgressInfo);
+	progress = ug_info_get (node->info, UgetProgressInfo);
 	if (progress && progress->total)
 		string = ug_str_from_int_unit (progress->total, NULL);
 	else
@@ -230,7 +230,7 @@ static void col_set_percent (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	progress = ug_data_get (node->data, UgetProgressInfo);
+	progress = ug_info_get (node->info, UgetProgressInfo);
 	if (progress && progress->total) {
 		string = ug_strdup_printf ("%d%c", progress->percent, '%');
 		g_object_set (cell, "visible", TRUE, NULL);
@@ -263,7 +263,7 @@ static void col_set_elapsed (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	progress = ug_data_get (node->data, UgetProgressInfo);
+	progress = ug_info_get (node->info, UgetProgressInfo);
 	if (progress)
 		string = ug_str_from_seconds ((int) progress->elapsed, TRUE);
 	else
@@ -292,10 +292,10 @@ static void col_set_left (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	progress = ug_data_get (node->data, UgetProgressInfo);
-	relation = ug_data_get (node->data, UgetRelationInfo);
+	progress = ug_info_get (node->info, UgetProgressInfo);
+	relation = ug_info_get (node->info, UgetRelationInfo);
 
-	if (progress && relation && relation->task.plugin)
+	if (progress && relation && relation->task)
 		string = ug_str_from_seconds ((int) progress->left, TRUE);
 	else
 		string = NULL;
@@ -322,10 +322,10 @@ static void col_set_speed (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	progress = ug_data_get (node->data, UgetProgressInfo);
-	relation = ug_data_get (node->data, UgetRelationInfo);
+	progress = ug_info_get (node->info, UgetProgressInfo);
+	relation = ug_info_get (node->info, UgetRelationInfo);
 
-	if (progress && relation && relation->task.plugin)
+	if (progress && relation && relation->task)
 		string = ug_str_from_int_unit (progress->download_speed, "/s");
 	else
 		string = NULL;
@@ -352,10 +352,10 @@ static void col_set_upload_speed (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	progress = ug_data_get (node->data, UgetProgressInfo);
-	relation = ug_data_get (node->data, UgetRelationInfo);
+	progress = ug_info_get (node->info, UgetProgressInfo);
+	relation = ug_info_get (node->info, UgetRelationInfo);
 
-	if (progress && relation && relation->task.plugin && progress->upload_speed)
+	if (progress && relation && relation->task && progress->upload_speed)
 		string = ug_str_from_int_unit (progress->upload_speed, "/s");
 	else
 		string = NULL;
@@ -381,7 +381,7 @@ static void col_set_uploaded (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	progress = ug_data_get (node->data, UgetProgressInfo);
+	progress = ug_info_get (node->info, UgetProgressInfo);
 	if (progress && progress->uploaded)
 		string = ug_str_from_int_unit (progress->uploaded, NULL);
 	else
@@ -408,7 +408,7 @@ static void col_set_ratio (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	progress = ug_data_get (node->data, UgetProgressInfo);
+	progress = ug_info_get (node->info, UgetProgressInfo);
 	if (progress && progress->ratio)
 		string = ug_strdup_printf ("%.2f", progress->ratio);
 	else
@@ -435,7 +435,7 @@ static void col_set_retry (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	common = ug_data_get (node->data, UgetCommonInfo);
+	common = ug_info_get (node->info, UgetCommonInfo);
 	if (common == NULL || common->retry_count == 0)
 		string = NULL;
 	else if (common->retry_count < 100)
@@ -467,7 +467,7 @@ static void col_set_category (GtkTreeViewColumn *tree_column,
 	node = node->base;
 
 	if (node->parent) {
-		common = ug_data_get(node->parent->data, UgetCommonInfo);
+		common = ug_info_get(node->parent->info, UgetCommonInfo);
 		if (common)
 			string = common->name;
 		else
@@ -496,7 +496,7 @@ static void col_set_uri (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	common = ug_data_get (node->data, UgetCommonInfo);
+	common = ug_info_get (node->info, UgetCommonInfo);
 	if (common)
 		string = common->uri;
 	else
@@ -522,7 +522,7 @@ static void col_set_added_on (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	ulog = ug_data_get (node->data, UgetLogInfo);
+	ulog = ug_info_get (node->info, UgetLogInfo);
 	if (ulog && ulog->added_time)
 		string = ug_str_from_time (ulog->added_time, FALSE);
 	else
@@ -549,7 +549,7 @@ static void col_set_completed_on (GtkTreeViewColumn *tree_column,
 		return;
 	node = node->base;
 
-	ulog = ug_data_get (node->data, UgetLogInfo);
+	ulog = ug_info_get (node->info, UgetLogInfo);
 	if (ulog && ulog->completed_time)
 		string = ug_str_from_time (ulog->completed_time, FALSE);
 	else
@@ -612,7 +612,7 @@ static void col_set_name_c (GtkTreeViewColumn *tree_column,
 //	}
 
 	node = node->base;
-	common = ug_data_get(node->data, UgetCommonInfo);
+	common = ug_info_get(node->info, UgetCommonInfo);
 	if (common && common->name)
 		name = common->name;
 	else
